 '***************************************************************************
' * Filename : BuyNowHelper.cs
' * Project  : Shareware Starter Kit: UIHelper
' * 
' * File Description 
' *  
' * The BuyNowHelper Class is used by the BuyNowForm to handle the interface with the Serverprocessing
' * class.  The server processing class handles the interaction with the web services.  BuyNowHelper
' * implements methods that the BuyNowForm calls.  
' * 
' * The method HandlePayment generates the Paypal URL based on variables set 
' * in APP.config and launchs a new IE window with the Paypal URL.  During this process a GUID is generated
' * to be used as the Paypal invoice number.  The invoice number is also returned to the caller so it can be used 
' * later on during the buy now web service processing to relate a Paypal transacation to a license request
' * 
' * The method Handleprocessing calls the Serverprocessing class and passes the invoice number as a parameter.  This initiates
' * the polling to the Buy Now Web Service.  When an answer or timout is received the UI is notified using the registered 
' * ProcessingFailed and ProcessingComplete events
' * 
' *  
' *
' * 
' * Implemented Namespaces
' *	SSK.SSKClient.UIHelper
' *
' *
' ***************************************************************************
Imports System
Imports System.Windows.Forms
Imports System.Configuration


Namespace SSK.SSKClient.UIHelper
   '/ <summary>
   '/ The BuyNowHelper Class is used by the BuyNowForm to handle the interface with the Serverprocessing
   '/ class.  The server processing class handles the interaction with the web services.  BuyNowHelper
   '/ implements methods that the BuyNowForm calls.  
   '/ 
   '/ The method HandlePayment generates the Paypal URL based on variables set 
   '/ in APP.config and launchs a new IE window with the Paypal URL.  During this process a GUID is generated
   '/ to be used as the Paypal invoice number.  The invoice number is also returned to the caller so it can be used 
   '/ later on during the buy now web service processing to relate a Paypal transacation to a license request
   '/ 
   '/ The method Handleprocessing calls the Serverprocessing class and passes the invoice number as a parameter.  This initiates
   '/ the polling to the Buy Now Web Service.  When an answer or timout is received the UI is notified using the registered 
   '/ ProcessingFailed and ProcessingComplete events
   '/ </summary>
   
   Public Class BuyNowHelper

      #Region "private class variables"
      
      'initialize the logger
      Private logger As New Logger(Application.StartupPath, "SskClientLog.txt")
      
      'invoice is a unique GUID used in the construction of the Paypal URL, and also in the polling of the buy now web service
      'it is used to relate a Paypal transaction with a license request on the buy now web service
      Private m_guidInvoice As System.Guid
      
      'this is used so we can pass the form handle through the serverProcessing class.
      'Because the web service calls are asynch, it is necessary to have this handle
      'so we can invoke the processingComplete and processingFailed events on the UI thread
      Private form As Form
      
      'server processing does all communication with the web services
      Private serverProcessing As Serverprocessing
      
      Private _requestedLicenseLevelPrice As String
      
      Private _requestedLicenseLevel As String
      
      Private m_strInvoice As String
      
      #End Region
      

#Region "Public event handlers"
        Public Event Processingcomplete(ByVal sender As Object, _
                                ByVal e As ProcessingCompletedEvent)

        Public Event Processingfailed(ByVal sender As Object, _
                                ByVal e As ProcessingFailedEvent)
#End Region
      #Region "constructor"
      
      '/ <summary>
      '/ The constructor initializes the invoice variable to a new System.Guid.NewGuid
      '/ and sets the private form variable
      '/ </summary>
      '/ <param name="formIn">A reference to the form using this class</param>
      Public Sub New(formin As Form)
            UIHelper.Logger.Log("BuyNowHelper : Constructor : Beginning")
            'set the private for the calling form
            form = formin

            'set invoice to a new GUID
            m_guidInvoice = System.Guid.NewGuid()

            UIHelper.Logger.Log("BuyNowHelper : Constructor : Completed")
      End Sub 'New
      #End Region
      
      #Region "public methods"
      
      
      Public Property RequestedLicenseLevelPrice() As String
         Get
            Return _requestedLicenseLevelPrice
         End Get
         Set
            _requestedLicenseLevelPrice = value
         End Set
      End Property
      
      
      Public Property RequestedLicenseLevel() As String
         Get
            Return _requestedLicenseLevel
         End Get
         Set
            _requestedLicenseLevel = value
         End Set
      End Property
      
      
      Public Property Invoice() As String
         Get
            Return m_strInvoice
         End Get
         Set
            m_strInvoice = value
         End Set
      End Property
      
      
      '/ <summary>
      '/ This method creates a new Paypal class and calls the HandleMethod method.  The Paypal
      '/ class generates the Paypal URL, which is based on the invoice passed in.  When HandlePayment
      '/ is called a new IE window is launched to this URL
      '/ </summary>
      Public Sub Handlepayment()
            UIHelper.Logger.Log("BuyNowHelper : Handlepayment : Beginning")

            'create Paypal class passing in invoice as string
            Dim paypal As New Paypal(m_guidInvoice.ToString(), _requestedLicenseLevelPrice, Me.form)

            'launch the new IE window to the Paypal URL
            paypal.Handlepayment()

            UIHelper.Logger.Log("BuyNowHelper : Handlepayment : Completed")
      End Sub 'Handlepayment
       
      
      
      '/ <summary>
      '/ This methos starts the polling for the buy now web service.  The calling form handle, invoice string,
      '/ and enumerated form type (defined in serverProcessing) are passed in.  The form handle is necessary so
      '/ we can invoke the event handlers on the UI thread from the asynch web service calls.  Handleprocessing also registers the BuyNowHelper
      '/ Event handlers with the signal processing class for ProcessingComplete and ProcessingFailed.
      '/ </summary>
      Public Sub Handleprocessing()
            UIHelper.Logger.Log("BuyNowHelper : Handleprocessing : Beginning")
            Dim buyNowInfo As New BuyNowInfo(_requestedLicenseLevel, m_guidInvoice.ToString(), Me.form, False)

            'create a new serverProcessing class passing in the calling form handle, invoice, and enum form type.
            serverProcessing = New Serverprocessing(buyNowInfo)

            'register the eventHandler ProcessingComplete of this class to the public event of

            AddHandler serverProcessing.Processingcomplete, AddressOf Me.SignalProcessingComplete

            'register the eventHandler ProcessingFailed of this class to the public event of
            'serverprocessing.  The message is passed in the args.  ProcessingFailedEventHandler is
            'defined in Serverprocessing
            AddHandler serverProcessing.Processingfailed, AddressOf Me.SignalProcessingFailed '

            'start the buyNowWeb service polling for a valid invoice
            serverProcessing.HandleBuyNow()


            UIHelper.Logger.Log("BuyNowHelper : Handleprocessing : Completed")
      End Sub 'Handleprocessing
      
      
      '/ <summary>
      '/ This function uses the serverProcessing class to process an invoice.  If all goes well we
      '/ will receive a valid ProductKey back from the web service and be activated.
      '/ </summary>
      '/ <param name="invoice">This is the invoice number entered by the user.</param>
        Public Sub HandleProcessingInvoice()

            UIHelper.Logger.Log("BuyNowHelper : HandleProcessingInvoice : Beginning")

            If m_strInvoice.Length > 0 AndAlso m_strInvoice <> " " Then
                Try

                    Dim buyNowInfo As New BuyNowInfo(_requestedLicenseLevel, m_strInvoice, Me.form, True)

                    'we pass in the invoice to send up in the web service, and also pass in
                    'the form handle from ActivateNowHelper.  This is used to invoke events on the Main UI thread.
                    'We pass in FormType so serverProcessing can know which constants to load depending on which 
                    'form is calling it.
                    serverProcessing = New Serverprocessing(buyNowInfo)

                    'register the eventHandler ProcessingComplete of this class to the public event of

                    AddHandler serverProcessing.Processingcomplete, AddressOf Me.SignalProcessingComplete

                    'register the eventHandler ProcessingFailed of this class to the public event of
                    'serverprocessing.  The message is passed in the args.  ProcessingFailedEventHandler is
                    'defined in Serverprocessing
                    AddHandler serverProcessing.Processingfailed, AddressOf Me.SignalProcessingFailed '

                    UIHelper.Logger.Log("BuyNowHelper : HandleProcessingInvoice : serverProcessing.HandleProcessing : Beginning")
                    'this call causes serverProcessing to launch the web service begin call
                    serverProcessing.HandleBuyNow()
                    UIHelper.Logger.Log("BuyNowHelper : HandleProcessingInvoice : serverProcessing.HandleProcessing : Completed")

                Catch ex As Exception
                    UIHelper.Logger.Log(("BuyNowHelper : HandleProcessingInvoice : Exception : " + ex.Message))
                    Dim message As String = "An Error occurred while contacting the server. "
                    Dim failureInfo As New FailureInfo
                    failureInfo.Message = message
                    failureInfo.Alert = UIHelper.FailureInfo.AlertType.None
                    SignalProcessingFailed(Me, New ProcessingFailedEvent(failureInfo, Me.form))
                End Try
            End If

            UIHelper.Logger.Log("BuyNowHelper : HandleProcessingInvoice : Completed")
        End Sub 'HandleProcessingInvoice


        '/ <summary>
        '/ This method must be called to dipose of the resources used by server processing.  
        '/ This is used when you want to close the form that initiated the begin processing call.
        '/ If this is not called the web service asynch call will generate an exception.
        '/ </summary> 
        Public Sub Cancelprocessing()
            System.Diagnostics.Debug.WriteLine("Calling UIHelper dispose")

            UIHelper.Logger.Log("BuyNowHelper : Cancelprocessing : Beginning")

            'we only want to do this if the serverProcessing class has been initialized
            If Not (serverProcessing Is Nothing) Then
                UIHelper.Logger.Log("BuyNowHelper : Cancelprocessing : serverProcessing.Cancelprocessing")
                serverProcessing.Cancelprocessing()
            End If

            UIHelper.Logger.Log("BuyNowHelper : Cancelprocessing : Completed")
        End Sub 'Cancelprocessing
      
      
      
      'This method fires this forms public event for processing complete.  This in turn
      'is registered with BuyNowForm
      Private Sub SignalProcessingComplete(sender As Object, e As ProcessingCompletedEvent)
            RaiseEvent Processingcomplete(Me, e)
      End Sub 'SignalProcessingComplete
      
      
      'This method fires this forms public event for processing failed.  This in turn
      'is registered with BuyNowForm
      Private Sub SignalProcessingFailed(sender As Object, e As ProcessingFailedEvent)
            RaiseEvent Processingfailed(Me, e)
      End Sub 'SignalProcessingFailed
      #End Region
      
      
      ' Implement IDispose
      Overloads Public Sub Dispose()
         Dispose(True)
         GC.SuppressFinalize(Me)
      End Sub 'Dispose
      
      
      Overloads Protected Overridable Sub Dispose(disposing As Boolean)
         If disposing Then
            ' Free other state (managed object)
            serverProcessing.Dispose()
            form.Dispose()
         End If
      End Sub 'Dispose
       ' Free your own state (unmanaged objects)
      ' Set large field to null
      
      ' Use C# destructor syntax for finalization code
        Protected Overloads Overrides Sub Finalize()
            Dispose(False)
            MyBase.Finalize()
        End Sub 'Finalize
   End Class 'BuyNowHelper 
   
   
   '/ <summary>
   '/ The class that contains all the information to be passed to the web service processing class
   '/ ServerProcessing
   '/ </summary>
   
   Public Class BuyNowInfo
      Private _licenseLevel As String
      Private _invoice As String
        Private _formReference As Form
        Private _isProductkey As Boolean
      
      
        Public Sub New(ByVal licenseLevel As String, ByVal invoice As String, ByVal form As Form, ByVal isProductKey As Boolean)
            _licenseLevel = licenseLevel
            _invoice = invoice
            _formReference = form
            _isProductkey = isProductKey
        End Sub 'New
      
      
      Public ReadOnly Property FormRef() As Form
         Get
            Return _formReference
         End Get
      End Property
      
      
      Public ReadOnly Property LicenseLevel() As String
         Get
            Return _licenseLevel
         End Get
      End Property
      
      
      Public ReadOnly Property Invoice() As String
         Get
            Return _invoice
         End Get
        End Property

        Public ReadOnly Property IsProductKey() As String
            Get
                Return _isProductkey
            End Get
        End Property

    End Class 'BuyNowInfo 
End Namespace 'SSK.SSKClient.UIHelper 