'***************************************************************************
' * Filename : DataProtection.cs
' * Project  : Shareware Starter Kit: UIHelper
' * 
' * File Description 
' *  
' * The DataProtector class uses DPAPI to encrypt and decrypt bytes of information.
' * This implementation can be viewed in detail on MSDN.
' * 
' * How to Create a DPAPI library- msdn.microsoft.com/library/ en-us/dnnetsec/html/SecNetHT07.asp
' * 
' * Implemented Namespaces
' *	SSK.SSKClient.UIHelper
' *
' *
' ***************************************************************************


'
'ToDo: Error processing original source shown below
'
'#region using directives
'-^--- expression expected
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Runtime.InteropServices

'
'ToDo: Error processing original source shown below
'using System.Runtime.InteropServices;
'#endregion
'-^--- expression expected

Namespace SSK.SSKClient.UIHelper

    Public Enum Store
        USE_MACHINE_STORE = 1
        USE_USER_STORE
    End Enum 'Store

    '/ <summary>
    '/ The dataprotector class encrypts and decrpyts data.
    '/ </summary>
    Public Class DataProtector

        <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Unicode)> _
        Friend Structure CRYPTPROTECT_PROMPTSTRUCT
            Public cbSize As Integer
            Public dwPromptFlags As Integer
            Public hwndApp As IntPtr
            Public szPrompt As String
        End Structure 'CRYPTPROTECT_PROMPTSTRUCT
        <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Unicode)> _
        Friend Structure DATA_BLOB
            Public cbData As Integer
            Public pbData As IntPtr
        End Structure 'DATA_BLOB

        <DllImport("Crypt32.dll", SetLastError:=True, CharSet:=System.Runtime.InteropServices.CharSet.Auto)> _
        Private Shared Function CryptProtectData(ByRef pDataIn As DATA_BLOB, ByVal szDataDescr As String, ByRef pOptionalEntropy As DATA_BLOB, ByVal pvReserved As IntPtr, ByRef pPromptStruct As CRYPTPROTECT_PROMPTSTRUCT, ByVal dwFlags As Integer, ByRef pDataOut As DATA_BLOB) As Boolean

        End Function

        <DllImport("Crypt32.dll", SetLastError:=True, CharSet:=System.Runtime.InteropServices.CharSet.Auto)> _
        Private Shared Function CryptUnprotectData(ByRef pDataIn As DATA_BLOB, ByVal szDataDescr As String, ByRef pOptionalEntropy As DATA_BLOB, ByVal pvReserved As IntPtr, ByRef pPromptStruct As CRYPTPROTECT_PROMPTSTRUCT, ByVal dwFlags As Integer, ByRef pDataOut As DATA_BLOB) As Boolean

        End Function

        <DllImport("kernel32.dll", CharSet:=System.Runtime.InteropServices.CharSet.Auto)> _
        Private Shared Function FormatMessage(ByVal dwFlags As Integer, ByRef lpSource As IntPtr, ByVal dwMessageId As Integer, ByVal dwLanguageId As Integer, ByRef lpBuffer As String, ByVal nSize As Integer, ByRef Arguments As IntPtr) As Integer
        End Function







        Private Shared NullPtr As IntPtr = CType(CInt(0), IntPtr)
        Private Const CRYPTPROTECT_UI_FORBIDDEN As Integer = &H1
        Private Const CRYPTPROTECT_LOCAL_MACHINE As Integer = &H4




        Private _store As Store


        '/ <summary>
        '/ Constructor for DataProtector Class
        '/ </summary>
        '/ <param name="tempStore"></param>
        Public Sub New(ByVal tempStore As Store)
            _store = tempStore
        End Sub 'New


        '/ <summary>
        '/ Encrypts an array of bytes.
        '/ </summary>
        '/ <param name="plainText">The byte array to be encrypted.</param>
        '/ <param name="optionalEntropy">Optionally used.  Not used in this implementation.</param>
        '/ <returns>The encrypted byte array</returns>
        Public Function Encrypt(ByVal plainText() As Byte, ByVal optionalEntropy() As Byte) As Byte()
            Dim retVal As Boolean = False

            Dim plainTextBlob As New DATA_BLOB()
            Dim cipherTextBlob As New DATA_BLOB()
            Dim entropyBlob As New DATA_BLOB()

            Dim prompt As New CRYPTPROTECT_PROMPTSTRUCT()
            InitPromptstruct(prompt)

            Dim dwFlags As Integer
            Try
                Try
                    Dim bytesSize As Integer = plainText.Length
                    plainTextBlob.pbData = Marshal.AllocHGlobal(bytesSize)
                    If IntPtr.Zero = plainTextBlob.pbData Then
                        Throw New Exception("Unable to allocate plaintext buffer.")
                    End If
                    plainTextBlob.cbData = bytesSize
                    Marshal.Copy(plainText, 0, plainTextBlob.pbData, bytesSize)
                Catch ex As Exception
                    Throw New Exception("Exception marshalling data. " + ex.Message)
                End Try
                If Store.USE_MACHINE_STORE = _store Then
                    'Using the machine store, should be providing entropy.
                    dwFlags = CRYPTPROTECT_LOCAL_MACHINE Or CRYPTPROTECT_UI_FORBIDDEN
                    'Check to see if the entropy is null
                    If Nothing Is optionalEntropy Then
                        'Allocate something
                        optionalEntropy = New Byte(-1) {}
                    End If
                    Try
                        Dim bytesSize As Integer = optionalEntropy.Length
                        entropyBlob.pbData = Marshal.AllocHGlobal(optionalEntropy.Length)
                        If IntPtr.Zero = entropyBlob.pbData Then
                            Throw New Exception("Unable to allocate entropy data buffer.")
                        End If
                        Marshal.Copy(optionalEntropy, 0, entropyBlob.pbData, bytesSize)
                        entropyBlob.cbData = bytesSize
                    Catch ex As Exception
                        Throw New Exception("Exception entropy marshalling data. " + ex.Message)
                    End Try
                Else 'Using the user store
                    dwFlags = CRYPTPROTECT_UI_FORBIDDEN
                End If
                retVal = CryptProtectData(plainTextBlob, "", entropyBlob, IntPtr.Zero, prompt, dwFlags, cipherTextBlob)
                If False = retVal Then
                    Throw New Exception("Encryption failed. " + GetErrorMessage(Marshal.GetLastWin32Error()))
                End If
            Catch ex As Exception
                Throw New Exception("Exception encrypting. " + ex.Message)
            End Try
            Dim cipherText(cipherTextBlob.cbData) As Byte
            Marshal.Copy(cipherTextBlob.pbData, cipherText, 0, cipherTextBlob.cbData)
            Return cipherText
        End Function 'Encrypt


        '/ <summary>
        '/ Decrypts an array of bytes
        '/ </summary>
        '/ <param name="cipherText">The byte array to decrypt.</param>
        '/ <param name="optionalEntropy">Optional.  Not used in this implementation.</param>
        '/ <returns>The unencrypted byte array.</returns>
        Public Function Decrypt(ByVal cipherText() As Byte, ByVal optionalEntropy() As Byte) As Byte()
            Dim retVal As Boolean = False
            Dim plainTextBlob As New DATA_BLOB()
            Dim cipherBlob As New DATA_BLOB()
            Dim prompt As New CRYPTPROTECT_PROMPTSTRUCT()
            InitPromptstruct(prompt)
            Try
                Try
                    Dim cipherTextSize As Integer = cipherText.Length
                    cipherBlob.pbData = Marshal.AllocHGlobal(cipherTextSize)
                    If IntPtr.Zero = cipherBlob.pbData Then
                        Throw New Exception("Unable to allocate cipherText buffer.")
                    End If
                    cipherBlob.cbData = cipherTextSize
                    Marshal.Copy(cipherText, 0, cipherBlob.pbData, cipherBlob.cbData)
                Catch ex As Exception
                    Throw New Exception("Exception marshalling data. " + ex.Message)
                End Try
                Dim entropyBlob As New DATA_BLOB()
                Dim dwFlags As Integer
                If Store.USE_MACHINE_STORE = _store Then
                    'Using the machine store, should be providing entropy.
                    dwFlags = CRYPTPROTECT_LOCAL_MACHINE Or CRYPTPROTECT_UI_FORBIDDEN
                    'Check to see if the entropy is null
                    If Nothing Is optionalEntropy Then
                        'Allocate something
                        optionalEntropy = New Byte(-1) {}
                    End If
                    Try
                        Dim bytesSize As Integer = optionalEntropy.Length
                        entropyBlob.pbData = Marshal.AllocHGlobal(bytesSize)
                        If IntPtr.Zero = entropyBlob.pbData Then
                            Throw New Exception("Unable to allocate entropy buffer.")
                        End If
                        entropyBlob.cbData = bytesSize
                        Marshal.Copy(optionalEntropy, 0, entropyBlob.pbData, bytesSize)
                    Catch ex As Exception
                        Throw New Exception("Exception entropy marshalling data. " + ex.Message)
                    End Try
                Else 'Using the user store
                    dwFlags = CRYPTPROTECT_UI_FORBIDDEN
                End If
                retVal = CryptUnprotectData(cipherBlob, Nothing, entropyBlob, IntPtr.Zero, prompt, dwFlags, plainTextBlob)
                If False = retVal Then
                    Throw New Exception("Decryption failed. " + GetErrorMessage(Marshal.GetLastWin32Error()))
                End If
                'Free the blob and entropy.
                If IntPtr.Zero <> cipherBlob.pbData Then
                    Marshal.FreeHGlobal(cipherBlob.pbData)
                End If
                If IntPtr.Zero <> entropyBlob.pbData Then
                    Marshal.FreeHGlobal(entropyBlob.pbData)
                End If
            Catch ex As Exception
                Throw New Exception("Exception decrypting. " + ex.Message)
            End Try
            Dim plainText(plainTextBlob.cbData) As Byte
            Marshal.Copy(plainTextBlob.pbData, plainText, 0, plainTextBlob.cbData)
            Return plainText
        End Function 'Decrypt


        Private Sub InitPromptstruct(ByRef ps As CRYPTPROTECT_PROMPTSTRUCT)
            ps.cbSize = Marshal.SizeOf(GetType(CRYPTPROTECT_PROMPTSTRUCT))
            ps.dwPromptFlags = 0
            ps.hwndApp = NullPtr
            ps.szPrompt = Nothing
        End Sub 'InitPromptstruct


        Private Shared Function GetErrorMessage(ByVal errorCode As Integer) As String
            Dim FORMAT_MESSAGE_ALLOCATE_BUFFER As Integer = &H100
            Dim FORMAT_MESSAGE_IGNORE_INSERTS As Integer = &H200
            Dim FORMAT_MESSAGE_FROM_SYSTEM As Integer = &H1000
            Dim messageSize As Integer = 255
            Dim lpMsgBuf As String = ""
            Dim dwFlags As Integer = FORMAT_MESSAGE_ALLOCATE_BUFFER Or FORMAT_MESSAGE_FROM_SYSTEM Or FORMAT_MESSAGE_IGNORE_INSERTS
            Dim ptrlpSource As New IntPtr()
            Dim prtArguments As New IntPtr()
            Dim retVal As Integer = FormatMessage(dwFlags, ptrlpSource, errorCode, 0, lpMsgBuf, messageSize, prtArguments)
            If 0 = retVal Then
                Throw New Exception("Failed to format message for error code " + errorCode + ". ")
            End If
            Return lpMsgBuf
        End Function 'GetErrorMessage
    End Class 'DataProtector 
End Namespace 'SSK.SSKClient.UIHelper 
