 '***************************************************************************
' * Filename : DataStore.cs
' * Project  : Shareware Starter Kit: ClientStorage
' * 
' * File Description 
' *  
' * The datastore is responsible for encrypting, storing, and retrieving application
' * license settings and configuration.
' * 
' *  
' *
' * 
' * Implemented Namespaces
' *	SSK.SSKClient.UIHelper
' *
' *
' ***************************************************************************

Imports System
Imports Microsoft.Win32 'used to access the registry
Imports System.IO 'used in isolated storage
Imports System.IO.IsolatedStorage 'used for isolated Storage
Imports System.Xml 'used to write and read the xml document.
Imports System.Xml.Serialization 'used to actually write the xml
Imports System.Text
Imports System.Configuration 'used to get the Trial key from the 
Imports System.Management 'used to create the unique machine ID.
Imports System.Runtime.Serialization
Imports System.Security.Cryptography
Imports System.Windows.Forms
Imports System.Runtime.Serialization.Formatters.Binary
Imports $safeprojectname$.SskWebService
Imports $safeprojectname$.SSK.SSKClient.ClientStorage
Imports $safeprojectname$.SSK.SSKClient.UIHelper


Namespace SSK.SSKClient.ClientStorage
   '/ <summary>
   '/ The DataStore class is used for storing and retrieving data
   '/ between each time the application runs.
   '/ It includes built in functions that are commonly needed by
   '/ shareware application developers.
   '/	
   '/	</summary>
   '/	<remarks>	
   '/The DataStore class works by using 
   '/<a href='http://msdn.microsoft.com/library/default.asp?url=/library/en-us/cpguide/html/cpconIsolatedStorage.asp'>
   '/isolated Storage</a> found in the .Net Framework.
   '/
   '/Each time the datastore is created it attempts to find the .dat file within Isolated storage.  If it cannot be found 
   '/the web service is contacted to pull down current product state and configuration.  This way, if the isolatedStorage
   '/location changes the state is still preserved at the server within the database.  
   
   Public Class DataStore

      '/ <summary>The name of the file written to isolated storage</summary>						
      Protected Const DATA_STORE_NAME As String = "DataStore.dat"
      
      #Region "private variables"
      'initialize the logger
        Private logger As New UIHelper.Logger(Application.StartupPath, "SskClientLog.txt")
      
      '/ <summary>
      '/ The one and only storage object. This is where all of the
      '/ information is cached after reading the isolated storage file. 
      '/ this object	is also the  one that is serialized when and written
      '/  to the isolated storage file.
      '/  <seealso cref='WriteDataToIsolatedStorage'/>
      '/ </summary>
      'private Storage storage;
      Private m_storage As $safeprojectname$.SskWebService.ClientAppConfig
      
      '/ <summary>
      '/ The private instance of Islated storage to use
      '/ </summary>
      Private isfs As IsolatedStorageFileStream
      
      '/ <summary>
      '/ If the web service request for the datastore fails we will set this flag to true
      '/ and return the appropriate product state and licensing information.
      '/ </summary>
      Private m_fRequestForDataStoreFailed As Boolean = False
      
      'a reference to the form that created this class
      Private formRef As Form
      
        'an event handler to signal a failure to the caller

        'Private __unknown As EventHandler
        'Private __unknown As __unknown
        'Private __unknown As ProcessingFailedEvent
        'Private __unknown As __unknown
        'Private __unknown As m_ehDataStoreFailed '
      'ToDo: Error processing original source shown below
      '        //an event handler to signal a failure to the caller
      '        private EventHandler<ProcessingFailedEvent> m_ehDataStoreFailed;
      '-----------------------------^--- Syntax error: 'identifier' expected
      '
      'ToDo: Error processing original source shown below
      '        //an event handler to signal a failure to the caller
      '        private EventHandler<ProcessingFailedEvent> m_ehDataStoreFailed;
      '---------------------------------------------------^--- Syntax error: 'identifier' expected
      '
      'ToDo: Error processing original source shown below
      '        //an event handler to signal a failure to the caller
      '        private EventHandler<ProcessingFailedEvent> m_ehDataStoreFailed;
      '------------------------------------------------------------------------^--- Syntax error: 'identifier' expected
      
      #End Region
      
      
      '/ <summary>
      '/ Handles Events that are used by the UI layer to signal a failed
      '/ completion for the Serverprocessing class.  The failure message
      '/ is passed pack in the event args.
      '/ </summary>
        'Public __unknown As EventHandler
        'Private __unknown As __unknown
        'Private __unknown As ProcessingFailedEvent
        'Private __unknown As __unknown
      
        'Property __unknown() As DataStoreFailed '
        'ToDo: Error processing original source shown below
        '        /// </summary>
        '        public EventHandler<ProcessingFailedEvent> DataStoreFailed
        '----------------------------^--- Syntax error: 'identifier' expected
        '
        'ToDo: Error processing original source shown below
        '        /// </summary>
        '        public EventHandler<ProcessingFailedEvent> DataStoreFailed
        '--------------------------------------------------^--- Syntax error: 'identifier' expected
        '
        'ToDo: Error processing original source shown below
        '        public EventHandler<ProcessingFailedEvent> DataStoreFailed
        '        {
        '---------^--- Syntax error: 'identifier' expected
        'Get
        '   Return m_ehDataStoreFailed
        'End Get
        'Set(ByVal value As DataStoreFailed)
        '    m_ehDataStoreFailed = Value
        'End Set
        'End Property

        Public Event DataStoreFailed(ByVal sender As Object, _
                        ByVal e As ProcessingFailedEvent)


        '/ <summary>
        '/ Default DataStore Constructor. It creates a new DataStore in isolated
        '/  storage if one is not found.
        '/ </summary>
        '/ <seealso cref='DefaultDataStoreValues'/>
        '/ <exception cref="DataStoreWriteException(string)"/>
        '/ <exception cref="DataStoreAppConfigException(string)"/>
        '/ <exception cref="DataStoreUniqueMachineIDException(string)"/>
        Public Sub New(ByVal form As Form)
            UIHelper.Logger.Log("DataStore : Constructor : Beginning")
            formRef = form
            m_storage = New $safeprojectname$.SskWebService.ClientAppConfig()
            UIHelper.Logger.Log("DataStore : Constructor : ReadDataFromIsolatedStorage : Beginning")
            ReadDataFromIsolatedStorage()
            UIHelper.Logger.Log("DataStore : Constructor : ReadDataFromIsolatedStorage : Completed")
            UIHelper.Logger.Log("DataStore : Constructor : Completed")
        End Sub 'New


        '/ <summary>
        '/ Returns the available license levels for this product.
        '/ </summary>

        Public ReadOnly Property AvailableLicenseLevels() As $safeprojectname$.SskWebService.LicenseLevelsPrice()
            Get
                Return m_storage.LicenseLevelPrices
            End Get
        End Property

        '/ <summary>
        '/ The length in ms to wait for a web service request to time out.
        '/ </summary>

        Public ReadOnly Property AbortTimerLength() As Double
            Get
                Return m_storage.AbortTimerLength
            End Get
        End Property

        '/ <summary>
        '/ The length in ms to wait before attempting to contact the web service again.
        '/ </summary>

        Public ReadOnly Property PollingTimerLength() As Double
            Get
                Return m_storage.PollingTimerLength
            End Get
        End Property

        '/ <summary>
        '/ The maximum number of times to contact the webservice before failing.
        '/ </summary>

        Public ReadOnly Property BuyNowMaxNumPoll() As Integer
            Get
                Return m_storage.BuyNowMaxNumPoll
            End Get
        End Property

        '/ <summary>
        '/ The payment setting used for PayPal configuration
        '/ </summary>

        Public ReadOnly Property PaymentSettings() As $safeprojectname$.SskWebService.PaymentSettings
            Get
                Return m_storage.PaymentSettings
            End Get
        End Property

        '/ <summary>
        '/ The maximum number of times to contact the server when the user is
        '/ entering an activation key.
        '/ </summary>

        Public ReadOnly Property ActivateNowMaxNumPoll() As Integer
            Get
                Return m_storage.ActivateNowMaxNumPoll
            End Get
        End Property

        '/ <summary>
        '/ The token that represents the developer when communicating with the
        '/ web service
        '/ </summary>

        Public ReadOnly Property Devtoken() As String
            Get
                Return m_storage.DevToken
            End Get
        End Property

        '/ <summary>
        '/ The name of this product used in communication with the seb service.
        '/ </summary>

        Public ReadOnly Property ProductName() As String
            Get
                Return m_storage.ProductName
            End Get
        End Property
        '/ <summary>
        '/ The unique identifier for this machine
        '/ </summary>

        Public ReadOnly Property UniqueMachineID() As String
            Get
                Return CreateUniqueMachineID()
            End Get
        End Property


        '/ <summary>
        '/ The unique combination of licenseLevel, client identity, and product name that is transmitted
        '/ to the server.  The returned license activation string is based on this value.
        '/ </summary>
        '/ <param name="licenseLevel">The requested license Level.</param>
        '/ <param name="product">The name of the product.</param>
        '/ <param name="clientIdentity">The unique identifier for the client.</param>
        '/ <returns>A hashed combination of these values.</returns>
        Public Function ClientInfo(ByVal licenseLevel As String, ByVal product As String, ByVal clientIdentity As String) As String
            UIHelper.Logger.Log("DataStore : ClientInfo : Beginning")
            Dim _clientInfo As String
            Dim uniEncoding As New UnicodeEncoding()
            Dim sha As New SHA1CryptoServiceProvider()
            Dim clientInfoByteArray() As Byte

            clientInfoByteArray = uniEncoding.GetBytes((licenseLevel + product + clientIdentity))


            _clientInfo = Convert.ToBase64String(sha.ComputeHash(clientInfoByteArray))
            sha.Clear()


            UIHelper.Logger.Log("DataStore : ClientInfo : Complete")
            Return _clientInfo
        End Function 'ClientInfo


        '/ <summary>
        '/ Sets the registrationKey received from the server.
        '/ </summary>
        '/ <param name="registrationKey">The registration key to set.</param>
        '/ <param name="licenseLevel">The license level the key is good for.</param>
        '/ <returns>True is the registration was verified and accepted, false otherwise.</returns>
        Public Function SetRegistrationKey(ByVal registrationKey As String, ByVal licenseLevel As String) As [Boolean]
            UIHelper.Logger.Log("DataStore : SetRegistrationKey : Beginning")
            Dim dataStoreSettings As $safeprojectname$.SskWebService.DataStore = m_storage.DataStoreSettings

            m_storage.DataStoreSettings.LicenseLevel = licenseLevel

            If VerifyLicense(registrationKey, licenseLevel, dataStoreSettings.UniqueMachineID) Then

                dataStoreSettings.RegistrationKey = registrationKey
                dataStoreSettings.LicenseLevel = licenseLevel
                m_storage.DataStoreSettings = dataStoreSettings
                WriteDataToIsolatedStorage()
                UIHelper.Logger.Log("DataStore : SetRegistrationKey : Completed : True")
                Return True
            Else
                UIHelper.Logger.Log("DataStore : SetRegistrationKey : Completed : False")
                Return False
            End If
        End Function 'SetRegistrationKey 
        '/ <summary>
        '/ Sets a flag that causes the client to contact the server for the latest data store the next
        '/ time the datastore is accessed.
        '/ </summary>

        Public Property RefreshDataStore() As Boolean

            Get
                Return m_storage.DataStoreSettings.RefreshDataStore
            End Get
            Set(ByVal value As Boolean)
                m_storage.DataStoreSettings.RefreshDataStore = True
            End Set
        End Property

        '/ <summary>
        '/ Returns the license level that the product is registered for.
        '/ </summary>
        '/ <returns>The licenseLevelsPrice object representing the appropriate license level</returns>

        Public ReadOnly Property GetLicenseLevel() As $safeprojectname$.SskWebService.LicenseLevelsPrice
            Get
                UIHelper.Logger.Log("DataStore : GetLicenseLevel : Beginning")

                Dim dataStore As $safeprojectname$.SskWebService.DataStore
                Dim licenseLevel = New LicenseLevelsPrice()
                licenseLevel.LicenseLevelID = 0
                licenseLevel.LicenseLevel = ""

                If Not (m_storage.DataStoreSettings Is Nothing) Then
                    dataStore = m_storage.DataStoreSettings
                Else
                    UIHelper.Logger.Log("DataStore : GetLicenseLevel : DataStoreSettings was null")
                    Return licenseLevel
                End If

                Dim licenseLevelsPrice As $safeprojectname$.SskWebService.LicenseLevelsPrice
                For Each licenseLevelsPrice In m_storage.LicenseLevelPrices
                    If VerifyLicense(dataStore.RegistrationKey, licenseLevelsPrice.LicenseLevel, dataStore.UniqueMachineID) Then
                        UIHelper.Logger.Log(("DataStore : GetLicenseLevel : Completed : LicenseLevel = " + licenseLevelsPrice.LicenseLevel))
                        Return licenseLevelsPrice
                    End If
                Next licenseLevelsPrice

                licenseLevel.LicenseLevel = "License Error"
                UIHelper.Logger.Log(("DataStore : GetLicenseLevel : Error : LicenseLevel = " + licenseLevel.LicenseLevel))
                Return licenseLevel
            End Get
        End Property

        '/ <summary>
        '/ Returns the enumerated state of the product depending on the activation and trial status.
        '/ Possibilities are Trial, ExpiredTrial, Activated, NoDataStore, and Error.
        '/ </summary>

        Public ReadOnly Property GetProductState() As ProductState

            Get
                UIHelper.Logger.Log("DataStore : GetProductState : Beginning")
                Dim trialLicenseLevel = New LicenseLevelsPrice()

                If Not (AvailableLicenseLevels Is Nothing) Then
                    Dim availableLicenseLevel As $safeprojectname$.SskWebService.LicenseLevelsPrice
                    For Each availableLicenseLevel In AvailableLicenseLevels
                        If availableLicenseLevel.IsTrialLicensekey Then
                            trialLicenseLevel = availableLicenseLevel
                            Exit For
                        End If
                    Next availableLicenseLevel

                    If GetLicenseLevel.LicenseLevel = trialLicenseLevel.LicenseLevel Then
                        If DaysElapsedSinceInstall > m_storage.TrialLengthDays Then
                            UIHelper.Logger.Log("DataStore : GetProductState : ProductState.ExpiredTrial")
                            Return ProductState.ExpiredTrial
                        Else
                            UIHelper.Logger.Log("DataStore : GetProductState : ProductState.Trial")
                            Return ProductState.Trial
                        End If
                    ElseIf GetLicenseLevel.LicenseLevel = "License Error" Then
                        UIHelper.Logger.Log("DataStore : GetProductState : ProductState.Error")
                        Return ProductState.Error
                    Else
                        UIHelper.Logger.Log("DataStore : GetProductState : ProductState.Activated")
                        Return ProductState.Activated
                    End If
                Else
                    If m_fRequestForDataStoreFailed Then
                        UIHelper.Logger.Log("DataStore : GetProductState : ProductState.NoDataStore")
                        Return ProductState.NoDataStore
                    Else
                        UIHelper.Logger.Log("DataStore : GetProductState : ProductState.Error")
                        Return ProductState.Error
                    End If
                End If
            End Get
        End Property


        '/ <summary>
        '/ Gets the number of the  days that have elapsed since the product was installed.
        '/ </summary>
        '/ <returns>The number of days elapsed since install.</returns>
        '/ <exception cref="DataStoreWriteException(string)"/>
        '/ <exception cref="DataStoreReadException(string)"/>
        '/ <exception cref="DataStoreChronologyException(string)">
        '/ If it appears that the date time has been set back this error will be thrown.
        '/ </exception>

        Public ReadOnly Property DaysElapsedSinceInstall() As Integer
            Get
                UIHelper.Logger.Log("DataStore : DaysElapsedSinceInstall : Beginning")
                'ReadDataFromIsolatedStorage();
                Dim dataStoreSettings As $safeprojectname$.SskWebService.DataStore = m_storage.DataStoreSettings

                If Not (dataStoreSettings Is Nothing) Then
                    Dim difference As TimeSpan 'difference between dataStoreStartDate and currentDate.									

                    'first check to see if the system time was set back.
                    If System.DateTime.Now < dataStoreSettings.StartDate Then
                        UIHelper.Logger.Log("DataStore : DaysElapsedSinceInstall : Error : DataStoreChronologyException")
                        Throw New DataStoreChronologyException("The current system time" + "is before the last known time stored in the isolated storage.")
                    End If


                    difference = System.DateTime.Now.Subtract(dataStoreSettings.StartDate)

                    If difference.Days < 0 Then
                        UIHelper.Logger.Log("DataStore : DaysElapsedSinceInstall : Error : DataStoreChronologyException")
                        'somehow the dataStore start date is set in the future...
                        Throw New DataStoreChronologyException("The DataStore Start Date is set in the future. " + "Either the System Time was set back, or even " + "more weirder the date stored in the data store" + "was set in the future.")
                    End If
                    UIHelper.Logger.Log(("DataStore : DaysElapsedSinceInstall : Completed : difference.Day =" + difference.Days.ToString()))
                    Return difference.Days
                    'there is no datastore so return a null date;
                Else
                    Return -1
                End If
            End Get
        End Property

        '/ <summary>
        '/ Returns a boolean value that determines if the product has been registered or not
        '/ </summary>
        '/ <returns>Boolean representing if product has been registered</returns>

        Public Property UserRegistered() As Boolean
            Get
                UIHelper.Logger.Log("DataStore : UserRegistered : Get : Beginning")
                'ReadDataFromIsolatedStorage();
                Dim dataStoreSettings As $safeprojectname$.SskWebService.DataStore = m_storage.DataStoreSettings
                If Not (dataStoreSettings Is Nothing) Then
                    UIHelper.Logger.Log(("DataStore : UserRegistered : Get : Completed : UserRegistered = " + dataStoreSettings.UserInfoRegistered.ToString()))
                    Return dataStoreSettings.UserInfoRegistered
                Else
                    UIHelper.Logger.Log("DataStore : UserRegistered : Get : datastore setting null : false")
                    Return False
                End If
            End Get
            Set(ByVal value As Boolean)
                UIHelper.Logger.Log("DataStore : UserRegistered : Set : Beginning")
                Dim dataStoreSettings As $safeprojectname$.SskWebService.DataStore = m_storage.DataStoreSettings
                dataStoreSettings.UserInfoRegistered = Value
                m_storage.DataStoreSettings = dataStoreSettings
                WriteDataToIsolatedStorage()
                UIHelper.Logger.Log("DataStore : UserRegistered : Set : Completed")
            End Set
        End Property


#Region "Private Functions and Protected members"

        '/ <summary>
        '/ Verifies the registration key against the machineID and licenseLevel included in the 
        '/ RegistrationKey
        '/ </summary>
        '/ <param name="registrationKey">The registration key from the server to validate</param>
        '/ <param name="licenseLevel">The license level to validate against</param>
        '/ <param name="uniqueMachineID">The machine ID to validate against</param>
        '/ <returns>True is a valid license, false is not valid for the given machine and license level</returns>
        Private Function VerifyLicense(ByVal registrationKey As String, ByVal licenseLevel As String, ByVal uniqueMachineID As String) As [Boolean]
            'if the license key has defaulted to the trial key return true
            Dim licenseLevelPrice As LicenseLevelsPrice
            For Each licenseLevelPrice In m_storage.LicenseLevelPrices
                If licenseLevelPrice.IsTrialLicensekey AndAlso licenseLevelPrice.LicenseLevel = registrationKey Then
                    Return True
                End If
            Next licenseLevelPrice

            Dim rsa As RSACryptoServiceProvider

            Dim authenticatedLicense() As Byte

            'validate the server public key to correct for possible typos
            If Not m_storage.ServerPublicKey.StartsWith("<RSAKeyValue><Modulus>") AndAlso Not m_storage.ServerPublicKey.EndsWith("</Exponent></RSAKeyValue>") Then
                m_storage.ServerPublicKey = "<RSAKeyValue><Modulus>" + m_storage.ServerPublicKey
                m_storage.ServerPublicKey += "</Modulus><Exponent>AQAB</Exponent></RSAKeyValue>"
            End If

            rsa = New RSACryptoServiceProvider()
            rsa.FromXmlString(m_storage.ServerPublicKey)


            authenticatedLicense = Convert.FromBase64String(registrationKey)

            If rsa.VerifyHash(Convert.FromBase64String(ClientInfo(licenseLevel, My.MySettings.Default.ProductName, uniqueMachineID)), CryptoConfig.MapNameToOID("SHA1"), authenticatedLicense) Then
                If licenseLevel = m_storage.DataStoreSettings.LicenseLevel Then
                    rsa.Clear()
                    Return True
                Else
                    rsa.Clear()
                    Return False
                End If
            Else
                rsa.Clear()
                Return False
            End If
        End Function 'VerifyLicense





        '/ <summary>
        '/ Reads all of the data in from the isolated storage
        '/ and stores it in the <see cref="storage"/> object.  If the isolated
        '/ storage file is not found it attempts to contact the web service to
        '/ initiate the data store.
        '/ </summary>
        '/ <exception cref="DataStoreAppConfigException(string)"/>
        '/ <exception cref="DataStoreUniqueMachineIDException(string)"/>
        '/ <exception cref="DataStoreWriteException(string)"/>
        '/ <exception cref="DataStoreReadException(string)"/>
        Protected Sub ReadDataFromIsolatedStorage()
            'load the stored data into the inner class so we have easy access to it.			
            'IsolatedStorageFile isf = IsolatedStorageFile.GetUserStoreForDomain();
            Dim isoStore As IsolatedStorageFile = IsolatedStorageFile.GetStore(IsolatedStorageScope.User Or IsolatedStorageScope.Assembly Or IsolatedStorageScope.Domain, Nothing, Nothing)

            Dim fileExists As Boolean = False

            Dim fileNames As String() = isoStore.GetFileNames(DATA_STORE_NAME)
            Dim file As String
            For Each file In fileNames
                If file = DATA_STORE_NAME Then
                    fileExists = True
                End If
            Next file


            If Not fileExists Then
                UIHelper.Logger.Log("DataStore : ReadDataFromIsolatedStorage : No DataStore- Contacting Server")

                Dim dataStoreInfo As New UIHelper.DataStoreInfo(My.Settings.DevToken, My.Settings.ProductName, CreateUniqueMachineID(), System.DateTime.Now, formRef)

                ContactServerForDefaultStorage(dataStoreInfo)
                WriteDataToIsolatedStorage()

            Else

                Try
                    read()

                    If m_storage.DataStoreSettings.RefreshDataStore Then
                        Dim dataStoreInfo As New UIHelper.DataStoreInfo(My.Settings.DevToken, My.Settings.ProductName, CreateUniqueMachineID(), System.DateTime.Now, formRef)
                        ContactServerToRefreshStorage(dataStoreInfo)
                        WriteDataToIsolatedStorage()
                    End If
                Catch ex As Exception
                    UIHelper.Logger.Log(("DataStore : ReadDataFromIsolatedStorage : Error reading store : " + ex.Message))
                    Dim dataStoreInfo As New UIHelper.DataStoreInfo(My.Settings.DevToken, My.Settings.ProductName, CreateUniqueMachineID(), System.DateTime.Now, formRef)
                    ContactServerForDefaultStorage(dataStoreInfo)
                    WriteDataToIsolatedStorage()
                End Try
            End If
        End Sub 'ReadDataFromIsolatedStorage


        Private Sub ContactServerForDefaultStorage(ByVal dataStoreInfo As UIHelper.DataStoreInfo)
            Try
                Dim sp As New UIHelper.Serverprocessing(dataStoreInfo)
                m_storage = sp.ContactServerForDefaultStorage(dataStoreInfo)
                UIHelper.Logger.Log("DataStore : ReadDataFromIsolatedStorage : Data Store Successfully Retrieved from server")
            Catch ex As Exception
                UIHelper.Logger.Log(("DataStore : ReadDataFromIsolatedStorage : Error retrieving Data Store from server : " + ex.Message))
            End Try

            If m_storage.DataStoreSettings Is Nothing Then
                Dim defaultConfig As New ClientAppConfig()
                defaultConfig.AbortTimerLength = My.Settings.AbortTimerLength
                defaultConfig.ActivateNowMaxNumPoll = 1
                defaultConfig.BuyNowMaxNumPoll = My.Settings.MaxPollNum

                '
                'ToDo: Error processing original source shown below
                '
                '                #region set default data store
                '-----------------^--- expression expected
                Dim defaultDataStore As New $safeprojectname$.SskWebService.DataStore()
                defaultDataStore.DateTime = System.DateTime.Now


                '
                'ToDo: Error processing original source shown below
                '
                '                #region set default license levels
                '-----------------^--- expression expected
                Dim licenseLevel1 As New LicenseLevelsPrice()
                licenseLevel1.IsTrialLicensekey = My.Settings.LicenseLevelIsTrialKey_1
                licenseLevel1.LicenseLevel = My.Settings.LicenseLevel_1
                licenseLevel1.LicenseLevelPrice = My.Settings.LicenseLevelPrice_1
                licenseLevel1.LicenseLevelID = 1
                licenseLevel1.Description = My.Settings.LicenseLevelDescription_1

                Dim licenseLevel2 As New LicenseLevelsPrice()
                licenseLevel2.IsTrialLicensekey = My.Settings.LicenseLevelIsTrialKey_2
                licenseLevel2.LicenseLevel = My.Settings.LicenseLevel_2
                licenseLevel2.LicenseLevelPrice = My.Settings.LicenseLevelPrice_2
                licenseLevel2.LicenseLevelID = 2
                licenseLevel2.Description = My.Settings.LicenseLevelDescription_2

                Dim licenseLevel3 As New LicenseLevelsPrice()
                licenseLevel3.IsTrialLicensekey = My.Settings.LicenseLevelIsTrialKey_3
                licenseLevel3.LicenseLevel = My.Settings.LicenseLevel_3
                licenseLevel3.LicenseLevelPrice = My.Settings.LicenseLevelPrice_3
                licenseLevel3.LicenseLevelID = 3
                licenseLevel3.Description = My.Settings.LicenseLevelDescription_3

                defaultConfig.LicenseLevelPrices = New LicenseLevelsPrice(2) {licenseLevel1, licenseLevel2, licenseLevel3}

                defaultDataStore.LicenseLevel = licenseLevel1.LicenseLevel
                Dim licenseLevelPrice As LicenseLevelsPrice
                For Each licenseLevelPrice In defaultConfig.LicenseLevelPrices
                    If licenseLevelPrice.IsTrialLicensekey Then
                        defaultDataStore.LicenseLevel = licenseLevelPrice.LicenseLevel
                        defaultDataStore.RegistrationKey = licenseLevelPrice.LicenseLevel
                    End If
                Next licenseLevelPrice


                '
                'ToDo: Error processing original source shown below
                '
                '                #endregion
                '-----------------^--- expression expected

                defaultDataStore.NumberOfTrialDaysElapsed = 0

                defaultDataStore.StartDate = System.DateTime.Now
                defaultDataStore.UniqueMachineID = UniqueMachineID
                defaultDataStore.UserInfoRegistered = False
                defaultDataStore.RefreshDataStore = True
                '
                'ToDo: Error processing original source shown below
                '                defaultDataStore.RefreshDataStore = true;
                '                #endregion
                '-----------------^--- expression expected

                defaultConfig.DataStoreSettings = defaultDataStore
                defaultConfig.DevToken = My.Settings.DevToken
                defaultConfig.LicenseLevelPrices = New LicenseLevelsPrice(2) {licenseLevel1, licenseLevel2, licenseLevel3}

                '
                'ToDo: Error processing original source shown below
                '
                '                #region set up default payment settings
                '-----------------^--- expression expected
                Dim paymentSettings As New PaymentSettings()
                paymentSettings.PaymentURL = My.Settings.PaymentURL
                paymentSettings.PayPalAccountEmail = My.Settings.PaymentAccountID
                paymentSettings.PayPalCancelURL = ""
                paymentSettings.PayPalCurrencyCode = My.Settings.PaymentCurrency
                paymentSettings.PayPalIncludeNote = False
                paymentSettings.PayPalItemNumber = My.Settings.PaymentItemNumber
                paymentSettings.PayPalLogoURL = ""
                paymentSettings.PayPalNoteTitle = ""
                paymentSettings.PayPalNotifyURL = My.Settings.PaymentNotifyURL
                paymentSettings.PayPalQuantity = 1
                paymentSettings.PayPalRequireShippingAddress = False
                paymentSettings.PayPalSuccessURL = ""
                paymentSettings.PayPalTag = ""
                defaultConfig.PaymentSettings = paymentSettings
                '
                'ToDo: Error processing original source shown below
                '                defaultConfig.PaymentSettings = paymentSettings;
                '                #endregion
                '-----------------^--- expression expected

                defaultConfig.PollingTimerLength = My.Settings.DefaultPollingInterval
                defaultConfig.ProductName = My.Settings.ProductName
                defaultConfig.ServerPublicKey = ""
                defaultConfig.TrialLengthDays = My.Settings.TrialLength

                m_storage = defaultConfig

                UIHelper.Logger.Log("DataStore : ReadDataFromIsolatedStorage : Initializing default datastore")
            End If
        End Sub 'ContactServerForDefaultStorage



        Private Sub ContactServerToRefreshStorage(ByVal dataStoreInfo As DataStoreInfo)
            UIHelper.Logger.Log("DataStore : ContactServerToRefreshStorage : Beginning")
            Try
                Dim sp As New Serverprocessing(dataStoreInfo)
                m_storage = sp.ContactServerForDefaultStorage(dataStoreInfo)
                UIHelper.Logger.Log("DataStore : ReadDataFromIsolatedStorage : Data Store Successfully Retrieved from server")
            Catch ex As Exception
                UIHelper.Logger.Log(("DataStore : ReadDataFromIsolatedStorage : Error retrieving Data Store from server : " + ex.Message))
            End Try

            UIHelper.Logger.Log("DataStore : ContactServerToRefreshStorage : Completed")
        End Sub 'ContactServerToRefreshStorage



        Private Sub read()

            'load the stored data into the inner class so we have easy access to it.			
            Dim isf As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForDomain()

            Dim dataProtector As New DataProtector(Store.USE_USER_STORE)

            Try
                isfs = New IsolatedStorageFileStream(DATA_STORE_NAME, FileMode.Open, FileAccess.Read, isf)
            Catch
                'Error while reading in the storage file
                Throw New DataStoreReadException("The inner exception was thrown while opening the islated storage file stream.")
            End Try

            'Read the file and put the data in our storage class.
            Try
                Dim byteBuffer(isfs.Length) As Byte

                While isfs.Position < isfs.Length
                    byteBuffer(isfs.Position) = CByte(isfs.ReadByte())
                End While


                byteBuffer = dataProtector.Decrypt(byteBuffer, Nothing)

                Dim memStream As New MemoryStream(byteBuffer)

                Dim xtr As New XmlTextReader(memStream)
                Dim xs As New XmlSerializer(GetType($safeprojectname$.SskWebService.ClientAppConfig))
                m_storage = CType(xs.Deserialize(xtr), $safeprojectname$.SskWebService.ClientAppConfig)
            Catch
                'Error while reading in the storage file
                Throw New DataStoreReadException("The inner exception was thrown while reading data from the isolated storage datastore.")
            Finally
                isfs.Close()
            End Try
        End Sub 'read




        '/ <summary>
        '/  Writes the storage object out to isolated storage
        '/  filename specified in: <see cref="DATA_STORE_NAME"/>
        '/ </summary>
        '/ <exception cref="DataStoreWriteException(string)"/>		
        Protected Sub WriteDataToIsolatedStorage()
            Try
                Dim memStream As New MemoryStream()

                Dim xs As New XmlSerializer(GetType($safeprojectname$.SskWebService.ClientAppConfig))
                xs.Serialize(memStream, m_storage)

                Dim dataProtector As New DataProtector(Store.USE_USER_STORE)
                Dim byteBuffer(memStream.Length) As Byte

                byteBuffer = memStream.GetBuffer()

                byteBuffer = dataProtector.Encrypt(byteBuffer, Nothing)



                Dim isf As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForDomain()
                Dim isfstream As New IsolatedStorageFileStream(DATA_STORE_NAME, FileMode.Create, FileAccess.Write, isf)

                isfstream.Write(byteBuffer, 0, byteBuffer.Length)

                isfstream.Close()

            Catch
                Throw New DataStoreWriteException("The inner exception was thrown while writing data to isolated storage.")
            End Try
        End Sub 'WriteDataToIsolatedStorage


        '/ <summary>
        '/ This creates a unique machine identifier, namely the first mac 
        '/ address found on the machine.
        '/ </summary>		
        '/ <returns>A string version of the mac address.</returns>
        '/ <exception cref="DataStoreUniqueMachineIDException(string)"/>
        Protected Function CreateUniqueMachineID() As String


            Try
                Dim cpuInfo As String = String.Empty
                Dim mc As New ManagementClass("Win32_Processor")
                Dim moc As ManagementObjectCollection = mc.GetInstances()
                Dim mo As ManagementObject
                For Each mo In moc
                    If cpuInfo = String.Empty Then
                        ' only return cpuInfo from first CPU
                        cpuInfo = mo.Properties("ProcessorId").Value.ToString()
                    End If
                Next mo
                Return cpuInfo
            Catch ex As Exception
                Throw New DataStoreUniqueMachineIDException("Error while calculating the unique machine ID.", ex)
            End Try
        End Function 'CreateUniqueMachineID

#End Region

#Region "Dispose of resources"

        ' Implement IDispose
        Public Overloads Sub Dispose()
            Dispose(True)
            GC.SuppressFinalize(Me)
        End Sub 'Dispose


        Protected Overridable Overloads Sub Dispose(ByVal disposing As Boolean)
            If disposing Then
            End If
        End Sub 'Dispose
        ' Free other state (managed object)
        ' Free your own state (unmanaged objects)

        ' Set large field to null

        ' Use C# destructor syntax for finalization code
        Protected Overloads Overrides Sub Finalize()
            Dispose(False)
            MyBase.Finalize()
        End Sub 'Finalize

#End Region
    End Class 'DataStore 
End Namespace 'SSK.SSKClient.ClientStorage




'
'ToDo: Error processing original source shown below
'
'    #region Enum ProductState
'-----^--- Syntax error: '}' expected
'/ <summary>
'/ Enumerated state for available product states {activated, Trial, Expiredtrial, Error}
'/ </summary>

Public Enum ProductState
   '/ <summary>
   '/ Enum state for a successfully activated product.
   '/ </summary>
   Activated
   
   '/ <summary>
   '/ Enum state for a product that is still in Trial mode (shareware).
   '/ </summary>
   Trial
   
   '/ <summary>
   '/ Enum state for a product that has not been activated, and the Trial
   '/ has expired
   '/ </summary>
   ExpiredTrial
   
   '/ <summary>
   '/ If there is no datastore.  This occurs when the client is unable to reach the web service
   '/ to initiailize the datastore.
   '/ </summary>
   NoDataStore
   
   '/ <summary>
   '/ Occurs when the product state is checked and the web service is still in progress.
   '/ </summary>
   WebServiceInProgress
   
   '/ <summary>
   '/ The product is in an unknown state.
   '/ </summary>
    [Error]
End Enum 'ProductState

'
'ToDo: Error processing original source shown below
'
'    #endregion
'-----^--- expression expected

'
'ToDo: Error processing original source shown below
'
'    #region Data Store exceptions
'-----^--- expression expected
'/ <remarks>
'/ Summary description for DataStoreWriteException.
'/ </remarks>>
<Serializable()>  _
Public Class DataStoreWriteException
   Inherits Exception
   
   Public Sub New()
   End Sub 'New
   
   
   '/ <summary>
   '/ Thrown when the DataStore cannot write.
   '/ </summary>
   '/ <param name="message"></param>
   Public Sub New(message As String)
      MyBase.New(message)
   End Sub 'New
   
   
   '/ <summary>
   '/ Thrown because another exception was thrown
   '/ during the datastore Write.
   '/ </summary>
   '/ <param name="message"></param>
   '/ <param name="inner"></param>
   Public Sub New(message As String, inner As Exception)
      MyBase.New(message, inner)
   End Sub 'New
   
   
   Protected Sub New(si As SerializationInfo, c As StreamingContext)
      MyBase.New(si, c)
   End Sub 'New
End Class 'DataStoreWriteException

'/ <remarks>
'/ Summary description for DataStoreReadException.
'/ </remarks>
<Serializable()>  _
Public Class DataStoreReadException
   Inherits Exception
   
   Public Sub New()
   End Sub 'New
   
   
   '/ <summary>
   '/ 
   '/ </summary>
   '/ <param name="message"></param>
   Public Sub New(message As String)
      MyBase.New(message)
   End Sub 'New
   
   '/ <summary>
   '/ 
   '/ </summary>
   '/ <param name="message"></param>
   '/ <param name="inner"></param>
   Public Sub New(message As String, inner As Exception)
      MyBase.New(message, inner)
   End Sub 'New
   
   
   Protected Sub New(si As SerializationInfo, c As StreamingContext)
      MyBase.New(si, c)
   End Sub 'New
End Class 'DataStoreReadException

'/ <remarks>
'/ Summary description for UniqueMachineIDException.
'/ </remarks>
<Serializable()>  _
Public Class DataStoreUniqueMachineIDException
   Inherits Exception
   
   Public Sub New()
   End Sub 'New
   
   
   '/ <summary>
   '/ 
   '/ </summary>
   '/ <param name="message"></param>
   Public Sub New(message As String)
      MyBase.New(message)
   End Sub 'New
   
   '/ <summary>
   '/ 
   '/ </summary>
   '/ <param name="message"></param>
   '/ <param name="inner"></param>
   Public Sub New(message As String, inner As Exception)
      MyBase.New(message, inner)
   End Sub 'New
   
   
   Protected Sub New(si As SerializationInfo, c As StreamingContext)
      MyBase.New(si, c)
   End Sub 'New
End Class 'DataStoreUniqueMachineIDException



'/ <remarks>
'/ Summary description for DataStoreChronologyException.
'/ </remarks>
<Serializable()>  _
Public Class DataStoreChronologyException
   Inherits Exception
   
   
   Public Sub New()
   End Sub 'New
   
   
   '/ <summary>
   '/ 
   '/ </summary>
   '/ <param name="message"></param>
   Public Sub New(message As String)
      MyBase.New(message)
   End Sub 'New
   
   
   '/ <summary>
   '/ 
   '/ </summary>
   '/ <param name="message"></param>
   '/ <param name="inner"></param>
   Public Sub New(message As String, inner As Exception)
      MyBase.New(message, inner)
   End Sub 'New
   
   
   Protected Sub New(si As SerializationInfo, c As StreamingContext)
      MyBase.New(si, c)
   End Sub 'New
End Class 'DataStoreChronologyException

'/ <remarks>
'/ Summary description for DataStoreAppConfigException.
'/ </remarks>
<Serializable()>  _
Public Class DataStoreAppConfigException
   Inherits Exception
   
   
   Public Sub New()
   End Sub 'New
   
   
   '/ <summary>
   '/ 
   '/ </summary>
   '/ <param name="message"></param>
   Public Sub New(message As String)
      MyBase.New(message)
   End Sub 'New
   
   
   Protected Sub New(si As SerializationInfo, c As StreamingContext)
      MyBase.New(si, c)
   End Sub 'New
   
   
   '/ <summary>
   '/ 
   '/ </summary>
   '/ <param name="message"></param>
   '/ <param name="inner"></param>
   Public Sub New(message As String, inner As Exception)
      MyBase.New(message, inner)
   End Sub 'New
End Class 'DataStoreAppConfigException


