 '***************************************************************************
' * Filename : PayPal.cs
' * Project  : Shareware Starter Kit: $safeprojectname$
' * 
' * File Description 
' *  
' * The PayPal class is responsible for generating the URL attributes required to open a web
' * browser window for PayPal payments.
' * 
' *
' * 
' * Implemented Namespaces
' *	SSK.SSKClient.UIHelper
' *
' 

Imports System
Imports System.Diagnostics
Imports System.Text
Imports System.Web
Imports System.Web.Configuration
Imports System.Windows.Forms
Imports $safeprojectname$.SSK.SSKClient.UIHelper
Imports $safeprojectname$.SSK.SSKClient.ClientStorage

Imports $safeprojectname$




Namespace SSK.SSKClient.UIHelper
   '/ <summary>
   '/  The PayPal class is responsible for generating the URL attributes required to open a web
   '/  browser window for PayPal payments.
   '/ </summary>
   
   Public Class Paypal
      
      Private _accountEmail As String
      Private _logoUrl As String
      Private _itemNumber As String
      Private _itemName As String
      Private _quantity As String
      Private _currencyCode As String
      Private _amount As String
      Private _tag As String
      Private _noteTitle As String
      Private _notifyUrl As String
      Private _successUrl As String
      Private _cancelUrl As String
      Private _requireShippingAddress As Boolean
      Private _includeNote As Boolean
      Private _invoice As String
      Private _devID As String
      Private dataStore As DataStore
      
      
      
      
      
      '/ <summary>
      '/ The constructor for the paypal URL generation class.
      '/ </summary>
      '/ <param name="tguid">The uniqueIdentifier to pass to payPal so we can
      '/ associate a transaction with a common known identity in the server database.</param>
      Public Sub New(tguid As String, amount As String, formRef As Form)
         
         dataStore = New DataStore(formRef)
         
         _accountEmail = dataStore.PaymentSettings.PayPalAccountEmail
         _logoUrl = dataStore.PaymentSettings.PayPalLogoURL
         _itemNumber = dataStore.PaymentSettings.PayPalItemNumber
         _quantity = dataStore.PaymentSettings.PayPalQuantity.ToString()
         _currencyCode = dataStore.PaymentSettings.PayPalCurrencyCode
         _tag = dataStore.PaymentSettings.PayPalTag
         _noteTitle = dataStore.PaymentSettings.PayPalNoteTitle
         _notifyUrl = dataStore.PaymentSettings.PayPalNotifyURL
         _successUrl = dataStore.PaymentSettings.PayPalSuccessURL
         _cancelUrl = dataStore.PaymentSettings.PayPalCancelURL
         _requireShippingAddress = dataStore.PaymentSettings.PayPalRequireShippingAddress
         _includeNote = dataStore.PaymentSettings.PayPalIncludeNote
            _devID = $safeprojectname$.My.Settings.DevToken
            _itemName = $safeprojectname$.My.Settings.ProductName
         _amount = amount
         
         _invoice = tguid
      End Sub 'New
       
      
      '/ <summary>
      '/ This creates a new IE window, generates the paypal URL, and passes
      '/ the URL to the window via the proc.StartInfo.Arguments.
      '/ </summary>
      Public Sub Handlepayment()
         
         'launch the IE window with the Paypal URL
         Dim proc As New Process()
         
         proc.StartInfo.FileName = "IExplore.exe"
         proc.StartInfo.Arguments = generatePayPalURL()
         proc.Start()
         proc.Dispose()
      End Sub 'Handlepayment
      
      #Region "generate the URL based on the private parameters"
      
      Private Function generatePayPalURL() As String
         
         Dim url As New StringBuilder()
         
         url.AppendFormat(dataStore.PaymentSettings.PaymentURL + "?cmd={0}&business={1}", "_xclick", HttpUtility.UrlEncode(_accountEmail))
         
         If Not String.IsNullOrEmpty(_logoUrl) Then
            url.AppendFormat("&image_url={0}", System.Web.HttpUtility.UrlEncode(_logoUrl))
         End If 
         If Not String.IsNullOrEmpty(_itemNumber) Then
            url.AppendFormat("&item_number={0}", HttpUtility.UrlEncode(_itemNumber))
         End If 
         If Not String.IsNullOrEmpty(_itemName) Then
            url.AppendFormat("&item_name={0}", HttpUtility.UrlEncode(_itemName))
         End If 
         If Not String.IsNullOrEmpty(_quantity) Then
            url.AppendFormat("&quantity={0}", HttpUtility.UrlEncode(_quantity))
         End If 
         If Not String.IsNullOrEmpty(_amount) Then
            url.AppendFormat("&amount={0}", HttpUtility.UrlEncode(_amount))
         End If 
         If Not String.IsNullOrEmpty(_currencyCode) Then
            url.AppendFormat("&currency_code={0}", HttpUtility.UrlEncode(_currencyCode))
         End If 
         If Not _requireShippingAddress Then
            url.Append("&no_shipping=1")
         End If 
         If Not _includeNote Then
            url.Append("&no_note=1")
         End If 
         If Not String.IsNullOrEmpty(_devID) Then
            url.AppendFormat("&custom={0}", HttpUtility.UrlEncode(_devID))
         End If 
         If Not String.IsNullOrEmpty(_invoice) Then
            url.AppendFormat("&invoice={0}", HttpUtility.UrlEncode(_invoice))
         End If 
         If Not String.IsNullOrEmpty(_notifyUrl) AndAlso _includeNote Then
            url.AppendFormat("&cn={0}", HttpUtility.UrlEncode(_noteTitle))
         End If 
         'not used in this implementation
         'if (BackgroundColor != BackgroundColor.Default)
         '	url.AppendFormat("&cs={0}", BackgroundColor == BackgroundColor.White ? "0" : "1");
         If Not String.IsNullOrEmpty(_notifyUrl) Then
            url.AppendFormat("&notify_url={0}", HttpUtility.UrlEncode(_notifyUrl))
         End If 
         If Not String.IsNullOrEmpty(_successUrl) Then
            url.AppendFormat("&return={0}", HttpUtility.UrlEncode(_successUrl))
         End If 
         If Not String.IsNullOrEmpty(_cancelUrl) Then
            url.AppendFormat("&cancel_return={0}", HttpUtility.UrlEncode(_cancelUrl))
         End If 
         Return url.ToString()
      End Function 'generatePayPalURL
      #End Region
   End Class 'Paypal
End Namespace 'SSK.SSKClient.UIHelper