 '***************************************************************************
' * Filename : Serverprocessing.cs
' * Project  : Shareware Starter Kit: UIHelper
' * 
' * File Description 
' * 
' *	Implements the layer of communications between the shareware application
' *  and the Shareware Service which allows the application user to 
' *  buy, register, and activate the application.
' * 
' * Implemented Namespaces
' * 
' *		SSK.SSKClient.UIHelper
' * 
' *
' ***************************************************************************
Imports System
Imports System.Windows.Forms
Imports System.Web.Services
Imports System.Web.Services.Protocols
Imports System.Configuration
Imports System.Net
Imports System.Xml
Imports $safeprojectname$.SskWebService
Imports System.Runtime.Serialization
Imports System.Reflection
Imports $safeprojectname$.SSK.SSKClient.ClientStorage



Namespace SSK.SSKClient.UIHelper


#Region "ProcessingFailed and ProcessingComplete custom event arguments"

    Public Class ProcessingFailedEvent
        Inherits EventArgs

        Public Sub New()
        End Sub 'New

        Public Sub New(ByVal str As String, ByVal form As Form)
            _message = str
            _formReference = form
        End Sub 'New


        Public Sub New(ByVal failure As FailureInfo, ByVal form As Form)
            _failureInfo = failure
            _formReference = form
        End Sub 'New


        Public Sub New(ByVal failure As FailureInfo)
            _failureInfo = failure
        End Sub 'New


        Public Sub New(ByVal ex As Exception)
            exception = ex
        End Sub 'New
        Private _failureInfo As FailureInfo

        Public Property FailureInformation() As FailureInfo
            Get
                Return _failureInfo
            End Get
            Set(ByVal value As FailureInfo)
                _failureInfo = value
            End Set
        End Property

        Private _message As String

        Public ReadOnly Property Message() As String
            Get
                Return _message
            End Get
        End Property

        Private _formReference As Form

        Public ReadOnly Property FormReference() As Form
            Get
                Return _formReference
            End Get
        End Property

        Private exception As Exception

        Public ReadOnly Property FailureException() As Exception
            Get
                Return exception
            End Get
        End Property
    End Class 'ProcessingFailedEvent


    Public Class ProcessingCompletedEvent
        Inherits EventArgs

        Public Sub New()
        End Sub 'New

        Public Sub New(ByVal argsObject As Object)
            _object = argsObject
        End Sub 'New


        Public Sub New(ByVal str As String, ByVal form As Form)
            _message = str
            _formReference = form
        End Sub 'New

        Private _message As String

        Public ReadOnly Property Message() As String
            Get
                Return _message
            End Get
        End Property

        Private _formReference As Form

        Public ReadOnly Property FormReference() As Form
            Get
                Return _formReference
            End Get
        End Property

        Private _object As Object

        Public ReadOnly Property ArgsObject() As [Object]
            Get
                Return _object
            End Get
        End Property
    End Class 'ProcessingCompletedEvent


#End Region


    'delegate to handle form invocation
    Public Delegate Sub MarshallThisFuncString(ByVal s As String)

    Public Delegate Function MarshallthisFuncString_RetDR(ByVal s As String) As DialogResult


    Public Delegate Sub MarshallThisFuncFailureInfo(ByVal f As FailureInfo)

    '/ <summary>
    '/ Implements the layer of the SSK that is used to communicate
    '/ with the Shareware Server.  It's called by the UI layer which is
    '/ either the RegisterForm, ActivateNowHelper, and the BuyNowHelper.
    '/ </summary>

    Public Class Serverprocessing


#Region "Private variables"

        'initialize the logger
        Private logger As New Logger(Application.StartupPath, "SskClientLog.txt")
        ' Keeps track of the number of times the BuyNow polling has timed out
        Private _timerCount As Integer
        ' How often we poll
        Private pollingTimer As New System.Timers.Timer()
        ' How much time to elapse before aborting an async Web Method
        Private abortTimer As New System.Timers.Timer()
        ' Object which manages the License Store
        Private m_datastore As SSK.SSKClient.ClientStorage.DataStore

        ' Tracks whether there is an Async Web Service method 
        ' currently in progress.
        Private wsInProgress As Boolean = False

        'used to ignore multiple returned results from the web service
        Private wsResultReceived As Boolean = False

        'keeps track of if the form that initially made the web service request
        'is open or closed
        Private formClosed As Boolean = False

        ' Reference to the Shareware Server Web Service
        Private m_webservice As New SharewareService()


        Private _info As Object

#End Region

#Region "Public event handlers"
        '/ <summary>
        '/ Handles Events that are used by the UI layer to signal a successful
        '/ completion for the Serverprocessing class.
        '/ </summary>

        Public Event Processingcomplete(ByVal sender As Object, _
                                ByVal e As ProcessingCompletedEvent)

        Public Event Processingfailed(ByVal sender As Object, _
                                ByVal e As ProcessingFailedEvent)

#End Region

#Region "Constructors"



        Public Sub New(ByVal dsInfo As DataStoreInfo)
            UIHelper.Logger.Log("ServerProcessing : Constructor (DataStoreInfo) : Beginning")
            m_webservice.UseDefaultCredentials = True

            _info = dsInfo
            UIHelper.Logger.Log("ServerProcessing : Constructor (DataStoreInfo) : Completed")
        End Sub 'New


        Public Sub New(ByVal fbInfo As FeedbackInfo)
            UIHelper.Logger.Log("ServerProcessing : Constructor (FeedbackInfo) : Beginning")
            m_datastore = New SSK.SSKClient.ClientStorage.DataStore(fbInfo.FormRef)
            m_webservice.UseDefaultCredentials = True

            abortTimer.Interval = m_datastore.AbortTimerLength
            abortTimer.AutoReset = False
            AddHandler abortTimer.Elapsed, AddressOf abortTimer_Elapsed

            _info = fbInfo
            UIHelper.Logger.Log("ServerProcessing : Constructor (FeedbackInfo) : Completed")
        End Sub 'New


        Public Sub New(ByVal eInfo As ExceptionInfo)
            UIHelper.Logger.Log("ServerProcessing : Constructor (ExceptionInfo) : Beginning")
            m_datastore = New SSK.SSKClient.ClientStorage.DataStore(eInfo.FormRef)
            m_webservice.UseDefaultCredentials = True

            abortTimer.Interval = m_datastore.AbortTimerLength
            abortTimer.AutoReset = False
            AddHandler abortTimer.Elapsed, AddressOf abortTimer_Elapsed

            _info = eInfo
            UIHelper.Logger.Log("ServerProcessing : Constructor (ExceptionInfo) : Completed")
        End Sub 'New


        Public Sub New(ByVal rInfo As RegisterInfo)
            UIHelper.Logger.Log("ServerProcessing : Constructor (RegisterInfo) : Beginning")
            m_datastore = New SSK.SSKClient.ClientStorage.DataStore(rInfo.FormRef)
            m_webservice.UseDefaultCredentials = True

            abortTimer.Interval = m_datastore.AbortTimerLength
            abortTimer.AutoReset = False
            AddHandler abortTimer.Elapsed, AddressOf abortTimer_Elapsed

            _info = rInfo
            UIHelper.Logger.Log("ServerProcessing : Constructor (RegisterInfo) : Completed")
        End Sub 'New


        Public Sub New(ByVal bnInfo As BuyNowInfo)
            UIHelper.Logger.Log("ServerProcessing : Constructor (BuyNowInfo) : Beginning")
            m_datastore = New SSK.SSKClient.ClientStorage.DataStore(bnInfo.FormRef)
            m_webservice.UseDefaultCredentials = True

            pollingTimer.Interval = m_datastore.PollingTimerLength
            pollingTimer.AutoReset = True

            abortTimer.Interval = m_datastore.AbortTimerLength
            abortTimer.AutoReset = False
            AddHandler abortTimer.Elapsed, AddressOf abortTimer_Elapsed

            _info = bnInfo
            UIHelper.Logger.Log("ServerProcessing : Constructor (BuyNowInfo) : Completed")
        End Sub 'New

#End Region

#Region "Web Service event handlers"


        '/ <summary>
        '/ Web service event handler for submit exception
        '/ </summary>
        '/ <param name="sender"></param>
        '/ <param name="args"></param>
        Sub _webservice_SubmitExceptionCompleted(ByVal sender As Object, ByVal args As SubmitExceptionCompletedEventArgs)
            UIHelper.Logger.Log("ServerProcessing : _webservice_SubmitExceptionCompleted")
            HandleSubmitExceptionCompleted(args)
        End Sub '_webservice_SubmitExceptionCompleted


        '/ <summary>
        '/ Web service event handler for issue/feedback submission
        '/ </summary>
        '/ <param name="sender"></param>
        '/ <param name="args"></param>
        Sub _webservice_SubmitIssueCompleted(ByVal sender As Object, ByVal args As SubmitIssueCompletedEventArgs)
            UIHelper.Logger.Log("ServerProcessing : _webservice_SubmitIssueCompleted")
            HandleSubmitIssueCompleted(args)
        End Sub '_webservice_SubmitIssueCompleted


        '/ <summary>
        '/ Web service event handler for user registration
        '/ </summary>
        '/ <param name="sender"></param>
        '/ <param name="args"></param>
        Sub _webservice_RegisterUserCompleted(ByVal sender As Object, ByVal args As RegisterUserCompletedEventArgs)
            UIHelper.Logger.Log("ServerProcessing : _webservice_RegisterUserCompleted")
            HandleRegisterUserCompleted(args)
        End Sub '_webservice_RegisterUserCompleted


        '/ <summary>
        '/ Web service event handler for obtaining client application configuration data
        '/ </summary>
        '/ <param name="sender"></param>
        '/ <param name="args"></param>
        Sub _webservice_GetClientAppConfigCompleted(ByVal sender As Object, ByVal args As GetClientAppConfigCompletedEventArgs)
            UIHelper.Logger.Log("ServerProcessing : _webservice_GetClientAppConfigCompleted")
            HandleGetClientAppConfigCompleted(CType(args, GetClientAppConfigCompletedEventArgs))
        End Sub '_webservice_GetClientAppConfigCompleted


        '/ <summary>
        '/ Web service event handler for getting a license.
        '/ </summary>
        '/ <param name="sender"></param>
        '/ <param name="args"></param>
        Sub _webservice_GetLicenseCompleted(ByVal sender As Object, ByVal args As GetLicenseCompletedEventArgs)
            UIHelper.Logger.Log("ServerProcessing : _webservice_GetLicenseCompleted")
            HandleGetLicenseCompleted(CType(args, GetLicenseCompletedEventArgs))
        End Sub '_webservice_GetLicenseCompleted
#End Region


        Public Sub HandleRegistration()
            UIHelper.Logger.Log("ServerProcessing : HandleRegistration : Beginning")
            Dim registerInfo As RegisterInfo = CType(_info, RegisterInfo)

            AddHandler m_webservice.RegisterUserCompleted, AddressOf _webservice_RegisterUserCompleted
            m_webservice.RegisterUserAsync(registerInfo.LastName, registerInfo.FirstName, registerInfo.Address, registerInfo.City, registerInfo.State, registerInfo.Zipcode, registerInfo.Phone, registerInfo.Email, registerInfo.DevToken, registerInfo.ProductName, m_datastore.UniqueMachineID, registerInfo.LicenseLevel)

            'start the abort timer to catch delayed responses from the web server.
            abortTimer.Start()

            UIHelper.Logger.Log("ServerProcessing : HandleRegistration : Completed")
        End Sub 'HandleRegistration


        Public Sub HandleFeedback()
            UIHelper.Logger.Log("ServerProcessing : HandleFeedback : Beginning")

            Dim feedbackInfo As FeedbackInfo = CType(_info, FeedbackInfo)

            AddHandler m_webservice.SubmitIssueCompleted, AddressOf _webservice_SubmitIssueCompleted
            m_webservice.SubmitIssueAsync(feedbackInfo.ProductName, feedbackInfo.LicenseLevel, feedbackInfo.Email, feedbackInfo.FirstName, feedbackInfo.LastName, feedbackInfo.Description, feedbackInfo.DevToken, m_datastore.UniqueMachineID)

            'start the abort timer to catch delayed responses from the web server.
            abortTimer.Start()

            UIHelper.Logger.Log("ServerProcessing : HandleFeedback : Completed")
        End Sub 'HandleFeedback


        Public Sub HandleException()
            UIHelper.Logger.Log("ServerProcessing : HandleException : Beginning")
            Dim exceptionInfo As ExceptionInfo = CType(_info, ExceptionInfo)

            AddHandler m_webservice.SubmitExceptionCompleted, AddressOf _webservice_SubmitExceptionCompleted
            m_webservice.SubmitExceptionAsync(exceptionInfo.Message, exceptionInfo.Source, exceptionInfo.StackTrace, exceptionInfo.MethodName, exceptionInfo.ProductName, exceptionInfo.DevToken, m_datastore.UniqueMachineID, exceptionInfo.LicenseLevel)

            'start the abort timer to catch delayed responses from the web server.
            abortTimer.Start()

            UIHelper.Logger.Log("ServerProcessing : HandleException : Completed")
        End Sub 'HandleException


        Public Sub HandleBuyNow()
            UIHelper.Logger.Log("ServerProcessing : HandleBuyNow : Beginning")

            AddHandler pollingTimer.Elapsed, AddressOf pollingTimer_Elapsed
            UIHelper.Logger.Log(("ServerProcessing : HandleProcessing : Polling Timer =" + pollingTimer.Interval.ToString() + " ms"))
            pollingTimer.Start()

            UIHelper.Logger.Log("ServerProcessing : HandleBuyNow : Completed")
        End Sub 'HandleBuyNow


        '/ <summary>
        '/ Start a transmission to the web service to get the DefaultStorage();
        '/ </summary>
        Public Function ContactServerForDefaultStorage(ByVal dataStoreInfo As DataStoreInfo) As ClientAppConfig
            UIHelper.Logger.Log("ServerProcessing : ContactServerForDefaultStorage : Beginning")
            UIHelper.Logger.Log(("ServerProcessing : ContactServerForDefaultStorage : WsInProgress : " + wsInProgress.ToString()))

            'If we are getting a new datastore and config settings from the server
            Return m_webservice.GetClientAppConfig(dataStoreInfo.ProductName, dataStoreInfo.DevId, dataStoreInfo.UniqueClientId, dataStoreInfo.StartDateTime)
        End Function 'ContactServerForDefaultStorage


        '/ <summary>
        '/ Start the transmission to the web service.
        '/ </summary>
        Private Sub ContactServerForLicensingInfo()
            UIHelper.Logger.Log("ServerProcessing : ContactServerForLicensingInfo : Beginning")
            UIHelper.Logger.Log(("ServerProcessing : ContactServerForLicensingInfo : WsInProgress : " + wsInProgress.ToString()))

            wsInProgress = True

            Dim buyNowInfo As BuyNowInfo = CType(_info, BuyNowInfo)

            AddHandler m_webservice.GetLicenseCompleted, AddressOf _webservice_GetLicenseCompleted
            m_webservice.GetLicenseAsync($safeprojectname$.My.Settings.ProductName, m_datastore.UniqueMachineID, buyNowInfo.LicenseLevel, buyNowInfo.Invoice, My.Settings.DevToken, m_datastore.UniqueMachineID)

            'start the abort timer to catch delayed responses from the web server.
            abortTimer.Start()

            UIHelper.Logger.Log("ServerProcessing : ContactServerForLicensingInfo : Completed")
        End Sub 'ContactServerForLicensingInfo



        '/ <summary>
        '/ Handle the client app config completed request received from the webservice
        '/ </summary>
        '/ <param name="eventArgs"></param>
        Private Sub HandleSubmitExceptionCompleted(ByVal submitExceptionArgs As SubmitExceptionCompletedEventArgs)
            UIHelper.Logger.Log("ServerProcessing::HandleSubmitExceptionCompleted - Beginning")

            abortTimer.Stop()

            Dim exceptionInfo As ExceptionInfo = CType(_info, ExceptionInfo)
            If Not Me.formClosed Then
                If submitExceptionArgs.Cancelled Then
                    UIHelper.Logger.Log("ServerProcessing : HandleSubmitExceptionCompleted : Web service was cancelled")
                    Dim cancelledInfo As FailureInfo = HandleWebServiceCancelled(submitExceptionArgs)
                    RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(cancelledInfo, exceptionInfo.FormRef))

                ElseIf Not (submitExceptionArgs.Error Is Nothing) Then
                    UIHelper.Logger.Log(("ServerProcessing : HandleSubmitExceptionCompleted : Error : " + CrackReason(submitExceptionArgs.Error)))
                    Dim failedInfo As FailureInfo = HandleWebServiceException(submitExceptionArgs.Error)
                    RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(failedInfo, exceptionInfo.FormRef))
                ElseIf submitExceptionArgs.Result Then
                    UIHelper.Logger.Log("ServerProcessing : HandleSubmitExceptionCompleted : Success")
                    Dim message As String = "Your error information was successfully submitted."
                    RaiseEvent Processingcomplete(Me, New ProcessingCompletedEvent(message, exceptionInfo.FormRef))
                Else
                    UIHelper.Logger.Log("ServerProcessing : HandleSubmitExceptionCompleted : Failed")
                    Dim failureInfo As New FailureInfo()
                    failureInfo.Message = "There was an error processing your exception submission."
                    failureInfo.Alert = failureInfo.AlertType.None
                    RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(failureInfo, exceptionInfo.FormRef))
                End If
            End If

            UIHelper.Logger.Log("ServerProcessing::HandleSubmitExceptionCompleted - End")
        End Sub 'HandleSubmitExceptionCompleted


        '/ <summary>
        '/ Handle the client app config completed request received from the webservice
        '/ </summary>
        '/ <param name="eventArgs"></param>
        Private Sub HandleSubmitIssueCompleted(ByVal submitIssueArgs As SubmitIssueCompletedEventArgs)
            UIHelper.Logger.Log("ServerProcessing::HandleSubmitIssueCompleted - Beginning")

            abortTimer.Stop()

            Dim feedbackInfo As FeedbackInfo = CType(_info, FeedbackInfo)
            If Not Me.formClosed Then
                If submitIssueArgs.Cancelled Then
                    UIHelper.Logger.Log("ServerProcessing : HandleSubmitIssueCompleted : Web service was cancelled")
                    Dim cancelledInfo As FailureInfo = HandleWebServiceCancelled(submitIssueArgs)
                    RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(cancelledInfo, feedbackInfo.FormRef))

                ElseIf Not (submitIssueArgs.Error Is Nothing) Then
                    UIHelper.Logger.Log(("ServerProcessing : HandleSubmitIssueCompleted : Error : " + CrackReason(submitIssueArgs.Error)))
                    Dim failedInfo As FailureInfo = HandleWebServiceException(submitIssueArgs.Error)
                    RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(failedInfo, feedbackInfo.FormRef))
                ElseIf submitIssueArgs.Result Then
                    UIHelper.Logger.Log("ServerProcessing : HandleSubmitIssueCompleted : Success")
                    Dim message As String = "Your feedback was successfully submitted."
                    RaiseEvent Processingcomplete(Me, New ProcessingCompletedEvent(message, feedbackInfo.FormRef))
                Else
                    UIHelper.Logger.Log("ServerProcessing : HandleSubmitIssueCompleted : Failed")
                    Dim failureInfo As New FailureInfo()
                    failureInfo.Message = "There was an error processing your feedback submission"
                    failureInfo.Alert = failureInfo.AlertType.None
                    RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(failureInfo, feedbackInfo.FormRef))
                End If
            End If

            UIHelper.Logger.Log("ServerProcessing::HandleSubmitIssueCompleted - End")
        End Sub 'HandleSubmitIssueCompleted


        '/ <summary>
        '/ Handle the client app config completed request received from the webservice
        '/ </summary>
        '/ <param name="eventArgs"></param>
        Private Sub HandleRegisterUserCompleted(ByVal registerArgs As RegisterUserCompletedEventArgs)
            UIHelper.Logger.Log("ServerProcessing::HandleRegisterUserCompleted - Beginning")

            abortTimer.Stop()

            Dim registerInfo As RegisterInfo = CType(_info, RegisterInfo)
            If Not Me.formClosed Then
                If registerArgs.Cancelled Then
                    UIHelper.Logger.Log("ServerProcessing : HandleRegisterUserCompleted : Registration Web service was cancelled")
                    Dim cancelledInfo As FailureInfo = HandleWebServiceCancelled(registerArgs)
                    RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(cancelledInfo, registerInfo.FormRef))
                ElseIf Not (registerArgs.Error Is Nothing) Then
                    UIHelper.Logger.Log(("ServerProcessing : HandleRegisterUserCompleted : Registration error : " + CrackReason(registerArgs.Error)))
                    Dim failedInfo As FailureInfo = HandleWebServiceException(registerArgs.Error)
                    RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(failedInfo, registerInfo.FormRef))
                ElseIf registerArgs.Result Then
                    UIHelper.Logger.Log("ServerProcessing : HandleRegisterUserCompleted : Success")

                    Dim message As String = "Your registration was successful."
                    m_datastore.UserRegistered = True
                    RaiseEvent Processingcomplete(Me, New ProcessingCompletedEvent(message, registerInfo.FormRef))
                Else
                    UIHelper.Logger.Log("ServerProcessing : HandleRegisterUserCompleted : Failed")
                    Dim failureInfo As New FailureInfo()
                    failureInfo.Message = "There was an error processing your registration request."
                    failureInfo.Alert = failureInfo.AlertType.None
                    RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(failureInfo, registerInfo.FormRef))
                End If
            End If

            UIHelper.Logger.Log("ServerProcessing::HandleRegisterUserCompleted - End")
        End Sub 'HandleRegisterUserCompleted


        '/ <summary>
        '/ Handle the client app config completed request received from the webservice
        '/ </summary>
        '/ <param name="eventArgs"></param>
        Private Sub HandleGetClientAppConfigCompleted(ByVal clientAppConfigArgs As GetClientAppConfigCompletedEventArgs)
            UIHelper.Logger.Log("ServerProcessing::HandleGetClientAppConfigCompleted - Beginning")

            abortTimer.Stop()

            Dim dsi As DataStoreInfo = CType(_info, DataStoreInfo)
            If clientAppConfigArgs.Cancelled Then
                UIHelper.Logger.Log("ServerProcessing : HandleGetClientAppConfigCompleted : Web service was cancelled")
                Dim cancelledInfo As FailureInfo = HandleWebServiceCancelled(clientAppConfigArgs)
                RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(cancelledInfo, dsi.FormRef))
            ElseIf Not (clientAppConfigArgs.Error Is Nothing) Then
                UIHelper.Logger.Log(("ServerProcessing : HandleGetClientAppConfigCompleted : Error : " + clientAppConfigArgs.Error.Message))
                Dim failedInfo As FailureInfo = HandleWebServiceException(clientAppConfigArgs.Error)
                RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(failedInfo, dsi.FormRef))

            ElseIf Not (clientAppConfigArgs.Result Is Nothing) Then
                UIHelper.Logger.Log("ServerProcessing : HandleGetClientAppConfigCompleted : Success")
                RaiseEvent Processingcomplete(Me, New ProcessingCompletedEvent(clientAppConfigArgs.Result))
            Else
                UIHelper.Logger.Log("ServerProcessing : HandleGetClientAppConfigCompleted : Failed")
                Dim failureInfo As New FailureInfo()
                failureInfo.Message = "There was a problem obtaining the configuration settings from the seb service."
                failureInfo.Alert = failureInfo.AlertType.None
                RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(failureInfo, dsi.FormRef))
            End If

            UIHelper.Logger.Log("ServerProcessing::HandleGetClientAppConfigCompleted - End")
        End Sub 'HandleGetClientAppConfigCompleted


        '/ <summary>
        '/ Handle the client app config completed request received from the webservice
        '/ </summary>
        '/ <param name="eventArgs"></param>
        Private Sub HandleGetLicenseCompleted(ByVal licenseArgs As GetLicenseCompletedEventArgs)
            UIHelper.Logger.Log("ServerProcessing::HandleGetLicenseCompleted - Beginning")

            abortTimer.Stop()
            pollingTimer.Stop()

            wsInProgress = False

            ' if we are already processing a complete, just bail.
            If wsResultReceived Then
                Return
            End If
            wsResultReceived = True

            Dim bni As BuyNowInfo = CType(_info, BuyNowInfo)
            If Not Me.formClosed Then
                If licenseArgs.Cancelled Then
                    UIHelper.Logger.Log("ServerProcessing : HandleGetLicenseCompleted : get license Web service was cancelled")
                    Dim cancelledInfo As FailureInfo = HandleWebServiceCancelled(licenseArgs)
                    RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(cancelledInfo, bni.FormRef))
                ElseIf Not (licenseArgs.Error Is Nothing) Then
                    UIHelper.Logger.Log(("ServerProcessing : HandleGetLicenseCompleted : get license Web service error: " + CrackReason(licenseArgs.Error)))
                    Dim failedInfo As FailureInfo = HandleWebServiceException(licenseArgs.Error)
                    RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(failedInfo, bni.FormRef))
                ElseIf Not (licenseArgs.Result Is Nothing) Then
                    UIHelper.Logger.Log("ServerProcessing : HandleGetLicenseCompleted :  get license Web service result received")

                    Dim result As New XmlDocument()
                    result.LoadXml(licenseArgs.Result)
                    'if (_ds.SetRegistrationKey(Convert.ToBase64String(result.SelectSingleNode("//AuthenticatedLicense")), result.SelectSingleNode("//LicenseLevel")))
                    If m_datastore.SetRegistrationKey(result.SelectSingleNode("//AuthenticatedLicense").InnerText, result.SelectSingleNode("//LicenseLevel").InnerText) Then
                        UIHelper.Logger.Log("ServerProcessing : HandleGetLicenseCompleted :  Set License Key Successful")
                        Dim message As String = "Activation Complete!  Please click below to register."
                        RaiseEvent Processingcomplete(Me, New ProcessingCompletedEvent(message, bni.FormRef))
                    Else
                        UIHelper.Logger.Log("ServerProcessing : HandleGetLicenseCompleted :  Set License Key Failed")
                        Dim failureInfo As New FailureInfo()
                        failureInfo.Message = "License response received from web service is invalid.  Please verify the server configuration."
                        failureInfo.Alert = failureInfo.AlertType.None
                        RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(failureInfo, bni.FormRef))
                    End If

                ElseIf licenseArgs.Result Is Nothing Then
                    UIHelper.Logger.Log("ServerProcessing : HandleGetLicenseCompleted : Null Result Received")
                    pollingTimer.Start()
                    UIHelper.Logger.Log("ServerProcessing : HandleGetLicenseCompleted : Polling Timer Started")
                    wsResultReceived = False
                    UIHelper.Logger.Log("ServerProcessing : HandleGetLicenseCompleted : wsResultReceived = false")
                End If
            End If

            UIHelper.Logger.Log("ServerProcessing::HandleGetLicenseCompleted - End")
        End Sub 'HandleGetLicenseCompleted


        Private Function HandleWebServiceCancelled(ByVal args As [Object]) As FailureInfo
            Dim failureInfo As New FailureInfo()

            failureInfo.Message = "An error occurred while contacting the web service. "
            failureInfo.Message += "The server did not respond in a timely manner.  Please verify your internet "
            failureInfo.Message += "connection and try again."

            Return failureInfo
        End Function 'HandleWebServiceCancelled


        Private Function HandleWebServiceException(ByVal ex As Exception) As FailureInfo
            Dim failureInfo As New FailureInfo()

            failureInfo.Message = CrackReason(ex)
            If Not (ex.InnerException Is Nothing) Then
                If ex.InnerException.GetType() Is GetType(System.Net.Sockets.SocketException) Then
                    failureInfo.Message = failureInfo.Message + " Please verify the following " + "URL is available using internet explorer: " + My.Settings.Client_SskWebService_SharewareService
                    failureInfo.Alert = failureInfo.AlertType.Port
                Else
                    failureInfo.Alert = failureInfo.AlertType.None
                End If
            Else
                failureInfo.Alert = failureInfo.AlertType.None
            End If

            Return failureInfo
        End Function 'HandleWebServiceException


        '/ <summary>
        '/ Looks inside an exception thrown at the SSK Server for
        '/ the reason why the exception was thrown.
        '/ </summary>
        '/ <param name="message"></param>
        '/ <returns>A string containing the reason message.</returns>
        Private Shared Function CrackReason(ByVal exception As Exception) As String

            Dim userMessage As String

            UIHelper.Logger.Log(("ServerProcessing : CrackReason : Crack Reason Raw Message : " + exception.Message))

            If exception.GetType() Is GetType(System.Net.Sockets.SocketException) Then
                Dim message As String = "There was an error contacting the server.  Please ensure you have "
                message += "internet connectivity or try again later."
                Return message
            End If

            If Not (exception.InnerException Is Nothing) Then
                If exception.InnerException.GetType() Is GetType(System.Net.Sockets.SocketException) Then
                    Dim message As String = "There was an error contacting the server.  Please ensure you have "
                    message += "internet connectivity or try again later."
                    Return message
                End If
            End If

            Dim startPos As Integer = exception.Message.IndexOf("Reason: '")


            startPos += 9

            Dim endPos As Integer = exception.Message.IndexOf("'", startPos)

            If endPos > 0 Then
                userMessage = exception.Message.Substring(startPos, endPos - startPos)
            Else
                userMessage = "An unknown error occurred while contacting the server." + exception.Message
            End If
            UIHelper.Logger.Log(("ServerProcessing : CrackReason : Crack Reason User Message : " + userMessage + " : Exception : " + exception.Message))

            Return userMessage
        End Function 'CrackReason 



#Region "AbortTimer code"

        '/ <summary>
        '/ The timer callback that stops the web service calls.
        '/ </summary>
        '/ <param name="sender"></param>
        '/ <param name="e"></param>
        Private Sub abortTimer_Elapsed(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs)
            UIHelper.Logger.Log("ServerProcessing : abortTimer_Elapsed : Abort timer elapsed")

            m_webservice.CancelAsync(Nothing)
        End Sub 'abortTimer_Elapsed 

#End Region

#Region "Web Service Polling code"

        '/ <summary>
        '/ This is the method that gets called when the Polling timer hits.  
        '/ It starts the Buy Now and Activate Now polling sequence.
        '/ </summary>
        '/ <param name="sender"></param>
        '/ <param name="e"></param>
        '/
        Private Sub pollingTimer_Elapsed(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs)
            UIHelper.Logger.Log("ServerProcessing : pollingTimer_Elapsed : Beginning")

            Dim maxNumPolls As Integer = m_datastore.BuyNowMaxNumPoll
            Dim buyNowInfo As BuyNowInfo = CType(_info, BuyNowInfo)

            UIHelper.Logger.Log(("ServerProcessing : pollingTimer_Elapsed : Timer Count:" + _timerCount.ToString() + " Max num polls:" + maxNumPolls.ToString()))

            If Not wsInProgress Then
                If _timerCount < maxNumPolls And Not (_timerCount = 1 And buyNowInfo.IsProductKey) Then

                    ContactServerForLicensingInfo()

                Else
                    UIHelper.Logger.Log("ServerProcessing : pollingTimer_Elapsed : PollingTimer.Stop")
                    pollingTimer.Stop()


                    Dim message As String = "Sorry, your invoice is not in the database. Please try again later."
                    UIHelper.Logger.Log("ServerProcessing : pollingTimer_Elapsed : max num polls exceeded")
                    Dim failureInfo As New FailureInfo
                    failureInfo.Message = message
                    failureInfo.Alert = UIHelper.FailureInfo.AlertType.None
                    RaiseEvent Processingfailed(Me, New ProcessingFailedEvent(failureInfo, buyNowInfo.FormRef))
                End If

                _timerCount += 1
            Else
                UIHelper.Logger.Log(("ServerProcessing : wsinprogress = " + wsInProgress.ToString() + " Did not contact server."))
            End If



            UIHelper.Logger.Log("ServerProcessing : pollingTimer_Elapsed : Completed")
        End Sub 'pollingTimer_Elapsed


#End Region


        '/ <summary>
        '/ Called by a UI helper class to stop a Web Service in situations
        '/ where the polling/abort system needs to be stopped.  This might
        '/ happen inside a BeginXXX and EndXXX call which means that 
        '/ we need to cancel the Async Web Service call.
        '/ </summary>
        Public Sub Cancelprocessing()
            UIHelper.Logger.Log("ServerProcessing : Cancelprocessing : Beginning")

            pollingTimer.Stop()
            formClosed = True
            UIHelper.Logger.Log("ServerProcessing : Cancelprocessing : Polling timer Stopped in cancel processing")

            ' Make sure that we abort the call only if there is currently
            ' an Async Web Service in progress.
            If wsInProgress = True Then
                wsInProgress = False
                abortTimer.Stop()

                'try to cancel the web service async call
                Try
                    m_webservice.CancelAsync(Nothing)
                Catch
                End Try

            End If

            UIHelper.Logger.Log("ServerProcessing : Cancelprocessing : Completed")
        End Sub 'Cancelprocessing


        ' Implement IDispose
        Public Overloads Sub Dispose()
            Dispose(True)
            GC.SuppressFinalize(Me)
        End Sub 'Dispose


        Protected Overridable Overloads Sub Dispose(ByVal disposing As Boolean)
            If disposing Then
                ' Free other state (managed object)
                m_datastore.Dispose()
                pollingTimer.Dispose()
                abortTimer.Dispose()
            End If
        End Sub 'Dispose
        ' Free your own state (unmanaged objects)
        ' Set large field to null

        ' Use C# destructor syntax for finalization code
        Protected Overloads Overrides Sub Finalize()
            Dispose(False)
            MyBase.Finalize()
        End Sub 'Finalize
    End Class 'Serverprocessing

    '/ <summary>
    '/ This class implements an exception which is thrown when an attempt is 
    '/ made to call a web service and there is a failure.
    '/ </summary>
    <Serializable()> _
    Public Class Webserviceconnectionexception
        Inherits Exception

        '
        ' Constructors
        '
        Public Sub New()
        End Sub 'New

        Public Sub New(ByVal s As String)
            MyBase.New(s)
        End Sub 'New

        Public Sub New(ByVal s As String, ByVal innerE As Exception)
            MyBase.New(s, innerE)
        End Sub 'New

        Protected Sub New(ByVal si As SerializationInfo, ByVal c As StreamingContext)
            MyBase.New(si, c)
        End Sub 'New
    End Class 'Webserviceconnectionexception

    '/ <summary>
    '/ This class implements an exception which is thrown when web service call is cancelled.
    '/ </summary>
    <Serializable()> _
    Public Class Webservicecancelledexception
        Inherits Exception

        '
        ' Constructors
        '
        Public Sub New()
        End Sub 'New

        Public Sub New(ByVal s As String)
            MyBase.New(s)
        End Sub 'New

        Public Sub New(ByVal s As String, ByVal innerE As Exception)
            MyBase.New(s, innerE)
        End Sub 'New

        Protected Sub New(ByVal si As SerializationInfo, ByVal c As StreamingContext)
            MyBase.New(si, c)
        End Sub 'New
    End Class 'Webservicecancelledexception

    '
    'ToDo: Error processing original source shown below
    '
    '    #region information classes
    '-----^--- expression expected
    '/ <summary>
    '/ ExceptionInfo holds all the information about an exception, and is passed into the web service processing
    '/ class- ServerProcessing.
    '/ </summary>

    Public Class ExceptionInfo
        Private _message As String
        Private _source As String
        Private _stacktrace As String
        Private _methodname As String
        Private _productname As String
        Private _devToken As String
        Private _uniqueMachineID As String
        Private _licenseLevel As String
        Private _formReference As Form


        Public Sub New(ByVal message As String, ByVal [source] As String, ByVal stackTrace As String, ByVal methodName As String, ByVal productName As String, ByVal devToken As String, ByVal uniqueMachineId As String, ByVal licenseLevel As String, ByVal form As Form)
            _message = message
            _source = [source]
            _stacktrace = stackTrace
            _methodname = methodName
            _productname = productName
            _devToken = devToken
            _uniqueMachineID = uniqueMachineId
            _licenseLevel = licenseLevel
            _formReference = form
        End Sub 'New


        Public ReadOnly Property FormRef() As Form
            Get
                Return _formReference
            End Get
        End Property


        Public ReadOnly Property Message() As String
            Get
                Return _message
            End Get
        End Property


        Public ReadOnly Property Source() As String
            Get
                Return _source
            End Get
        End Property


        Public ReadOnly Property StackTrace() As String
            Get
                Return _stacktrace
            End Get
        End Property


        Public ReadOnly Property MethodName() As String
            Get
                Return _methodname
            End Get
        End Property

        Public ReadOnly Property ProductName() As String
            Get
                Return _productname
            End Get
        End Property


        Public ReadOnly Property DevToken() As String
            Get
                Return _devToken
            End Get
        End Property


        Public ReadOnly Property UniqueMachineID() As String
            Get
                Return _uniqueMachineID
            End Get
        End Property


        Public ReadOnly Property LicenseLevel() As String
            Get
                Return _licenseLevel
            End Get
        End Property
    End Class 'ExceptionInfo


    '/ <summary>
    '/ The class containing all feedback information that is passed to the web service processing class
    '/ ServerProcessing for transmission to the server.
    '/ </summary>

    Public Class FeedbackInfo
        Private _productname As String
        Private _licenselevel As String
        Private _email As String
        Private _firstname As String
        Private _lastname As String
        Private _description As String
        Private _devToken As String
        Private _formReference As Form



        Public Sub New(ByVal productName As String, ByVal licenseLevel As String, ByVal email As String, ByVal firstName As String, ByVal lastName As String, ByVal description As String, ByVal devtoken As String, ByVal form As Form)
            _productname = productName
            _licenselevel = licenseLevel
            _email = email
            _firstname = firstName
            _lastname = lastName
            _description = description
            _devToken = devtoken
            _formReference = form
        End Sub 'New 


        Public ReadOnly Property FormRef() As Form
            Get
                Return _formReference
            End Get
        End Property


        Public ReadOnly Property ProductName() As String
            Get
                Return _productname
            End Get
        End Property


        Public ReadOnly Property LicenseLevel() As String
            Get
                Return _licenselevel
            End Get
        End Property

        Public ReadOnly Property Email() As String
            Get
                Return _email
            End Get
        End Property

        Public ReadOnly Property FirstName() As String
            Get
                Return _firstname
            End Get
        End Property

        Public ReadOnly Property LastName() As String
            Get
                Return _lastname
            End Get
        End Property

        Public ReadOnly Property Description() As String
            Get
                Return _description
            End Get
        End Property

        Public ReadOnly Property DevToken() As String
            Get
                Return _devToken
            End Get
        End Property
    End Class 'FeedbackInfo
    '/ <summary>
    '/ The class that contains all registration information to be transmitted to the server.
    '/ </summary>

    Public Class RegisterInfo

        Private _productname As String
        Private _licenselevel As String
        Private _email As String
        Private _firstname As String
        Private _lastname As String
        Private _devToken As String
        Private _address As String
        Private _city As String
        Private _state As String
        Private _zipcode As String
        Private _phone As String
        Private _uniqueMachineID As String
        Private _formReference As Form



        Public Sub New(ByVal productName As String, ByVal licenseLevel As String, ByVal email As String, ByVal firstName As String, ByVal lastName As String, ByVal devtoken As String, ByVal address As String, ByVal city As String, ByVal state As String, ByVal zipcode As String, ByVal phone As String, ByVal uniqueMachineID As String, ByVal form As Form)
            _productname = productName
            _licenselevel = licenseLevel
            _email = email
            _firstname = firstName
            _lastname = lastName
            _devToken = devtoken
            _address = address
            _city = city
            _state = state
            _zipcode = zipcode
            _phone = phone
            _uniqueMachineID = uniqueMachineID
            _formReference = form
        End Sub 'New 


        Public ReadOnly Property FormRef() As Form
            Get
                Return _formReference
            End Get
        End Property


        Public ReadOnly Property ProductName() As String
            Get
                Return _productname
            End Get
        End Property


        Public ReadOnly Property LicenseLevel() As String
            Get
                Return _licenselevel
            End Get
        End Property

        Public ReadOnly Property Email() As String
            Get
                Return _email
            End Get
        End Property

        Public ReadOnly Property FirstName() As String
            Get
                Return _firstname
            End Get
        End Property

        Public ReadOnly Property LastName() As String
            Get
                Return _lastname
            End Get
        End Property

        Public ReadOnly Property DevToken() As String
            Get
                Return _devToken
            End Get
        End Property

        Public ReadOnly Property Address() As String
            Get
                Return _address
            End Get
        End Property

        Public ReadOnly Property City() As String
            Get
                Return _city
            End Get
        End Property

        Public ReadOnly Property State() As String
            Get
                Return _state
            End Get
        End Property

        Public ReadOnly Property Zipcode() As String
            Get
                Return _zipcode
            End Get
        End Property

        Public ReadOnly Property Phone() As String
            Get
                Return _phone
            End Get
        End Property

        Public ReadOnly Property UniqueMachineID() As String
            Get
                Return _uniqueMachineID
            End Get
        End Property
    End Class 'RegisterInfo 


    Public Class FailureInfo

        Public Enum AlertType
            Port
            None
        End Enum 'AlertType

        Public Sub New()
        End Sub 'New

        Private _message As String

        Public Property Message() As String
            Get
                Return _message
            End Get
            Set(ByVal value As String)
                _message = value
            End Set
        End Property

        Private _alert As AlertType

        Public Property Alert() As AlertType
            Get
                Return _alert
            End Get
            Set(ByVal value As AlertType)
                _alert = value
            End Set
        End Property

        Private _webServiceException As WebException

        Public Property WebServiceException() As WebException
            Get
                Return _webServiceException
            End Get
            Set(ByVal value As WebException)
                _webServiceException = value
            End Set
        End Property
    End Class 'FailureInfo

    '/ <summary>
    '/ Class that contains specifics about the datastore.
    '/ </summary>

    Public Class DataStoreInfo
        Private _devId As String
        Private _productname As String
        Private _uniqueClientId As String
        Private _dateTime As DateTime
        Private _form As Form


        Public Sub New(ByVal devId As String, ByVal productName As String, ByVal uniqueClientId As String, ByVal startDateTime As DateTime, ByVal form As Form)
            _devId = devId
            _productname = productName
            _uniqueClientId = uniqueClientId
            _dateTime = startDateTime
            _form = form
        End Sub 'New

        Public ReadOnly Property FormRef() As Form
            Get
                Return _form
            End Get
        End Property

        Public ReadOnly Property StartDateTime() As DateTime
            Get
                Return _dateTime
            End Get
        End Property


        Public ReadOnly Property DevId() As String
            Get
                Return _devId
            End Get
        End Property


        Public ReadOnly Property ProductName() As String
            Get
                Return _productname
            End Get
        End Property


        Public ReadOnly Property UniqueClientId() As String
            Get
                Return _uniqueClientId
            End Get
        End Property
    End Class 'DataStoreInfo 

End Namespace 'SSK.SSKClient.UIHelper