 '===============================================================================
' Microsoft User Interface Process Application Block for .NET Quick Start
' http://msdn.microsoft.com/library/en-us/dnbda/html/uip.asp
'
' FormValidator.cs
'
' This file contains the implementations of the FormValidator class.
'
' 
'===============================================================================
' Copyright (C) 2000-2001 Microsoft Corporation
' All rights reserved.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
' OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
' LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR
' FITNESS FOR A PARTICULAR PURPOSE.
'==============================================================================
Imports System
Imports System.Windows.Forms


Namespace SSK.SSKClient.SSKForms
   '/ <remarks>
   '/ 
   '/ </remarks>
   
   NotInheritable Public Class FormValidator
      
      '/ <summary>
      '/ Delegate that defines a method that can perform validation against a control
      '/ </summary>
      Delegate Function ControlValidator(control As Control) As Boolean
      
      
      
      Private Sub New()
      End Sub 'New
      
      
      
      '/ <summary>
      '/ Method that determines whether the contents of a field are valid
      '/ </summary>
      '/ <param name="errNotifier">ErrorProvider component that will be used to provide Error notification</param>
      '/ <param name="control">Control that is being validated ( also the one that the ErrorProvider component will
      '/ render itself next to if an Error occurs.</param>
      '/ <param name="errMessage">String containing the Error message to show if the validation fails</param>
      '/ <param name="validator">Delegate that points to a method that will perform the validation</param>
      '/ <returns></returns>
      Public Shared Function Fieldisvalid(errnotifier As ErrorProvider, control As Control, errmessage As String, validator As ControlValidator) As Boolean
         
         Dim isvalid As Boolean = False
         
         ' invoke the method that will perform the validation against the control
         If Not validator(control) Then
            ' if the validation did not pass then set the Error message next to the control
            errnotifier.SetError(control, errmessage)
            isvalid = False
         Else
            ' clear the Error message associated with the control
            errnotifier.SetError(control, "")
            isvalid = True
         End If
         
         Return isvalid
      End Function 'Fieldisvalid
      
      
      
      '/ <summary>
      '/ Method that matches the ControlValidator signature and will perform a check to see if the text property of a control
      '/ has been set to anything other than a zero length string
      '/ </summary>
      '/ <param name="control"></param>
      '/ <returns></returns>
      Public Shared Function Hastextvalidator(control As Control) As Boolean
         Return control.Text.Trim().Length > 0
      End Function 'Hastextvalidator
      
      '/ <summary>
      '/ 
      '/ </summary>
      '/ <returns></returns>
      Public Shared Function Isnumericvalidator() As Boolean
         Return True
      End Function 'Isnumericvalidator
      
      '/ <summary>
      '/		  validate the email form entry
      '/ </summary>
      '/ <param name="control"></param>
      '/ <returns></returns>
      Public Shared Function Isemailvalidator(control As Control) As Boolean
         If control.Text.Length <= 0 Then
            Return False
         Else
            Return control.Text.IndexOf("@", 0, control.Text.Length) >= 0
         End If
      End Function 'Isemailvalidator
   End Class 'FormValidator
   
   '/ <remarks>
   '/ Class that handles control validation results
   '/ </remarks>
   
   Public Class Validationresult
      Private _isvalid As Boolean
      Private _errormessage As String
      
      
      '/ <summary>
      '/ The constructor for the ValidationResult slass
      '/ </summary>
      '/ <param name="isValid">Boolean for whether the control has valid input or not.</param>
      '/ <param name="errorMessage">The Error message to append to the control err notifier.</param>
      Public Sub New(isvalid As Boolean, errormessage As String)
         Me._isvalid = isvalid
         Me._errormessage = errormessage
      End Sub 'New
      
      '/ <summary>
      '/ Get the control Error message.
      '/ </summary>
      '/ <value>The Error message for the control.</value>
      
      Public ReadOnly Property Errormessage() As String
         Get
            Return _errormessage
         End Get
      End Property
      
      '/ <summary>
      '/ Get the control valid status.
      '/ </summary>
      '/ <value>The status of the control.</value>
      
      Public ReadOnly Property Isvalid() As Boolean
         Get
            Return _isvalid
         End Get
      End Property
   End Class 'Validationresult 
End Namespace 'SSK.SSKClient.SSKForms 