 'Shareware Starter Kit --SplashForm
'Summary:  This is the default form provided for the splash page.
'Author: Brian Trautman briant@personifydesign.com
'Date Created: 4/11/2004
'Date Last Modified: 4/13/2005
'
'ToDo: Error processing original source shown below
'
'#region using directives
'-^--- expression expected
Imports System
Imports System.Drawing
Imports System.Collections
Imports System.ComponentModel
Imports System.Windows.Forms
Imports System.Drawing.Drawing2D
Imports System.Threading


'
'ToDo: Error processing original source shown below
'using SSK.SSKClient.ClientStorage;
'#endregion
'-^--- expression expected

Namespace SSK.SSKClient.SSKForms
   '/ <summary>
   '/ Summary description for SplashForm.
   '/ </summary>
   
   Public Class SplashForm
      Inherits System.Windows.Forms.Form
      '/ <summary>
      '/ Required designer variable.
      '/ </summary>
      Private components As System.ComponentModel.Container = Nothing
      
      'the timer used to close the splash form and turn over control to the main form on event
      Private timer As System.Windows.Forms.Timer
      
      'the timer used to fade in and out the splash form
      Private fadetimer As System.Windows.Forms.Timer
      
      'this is the variable that determines the fade increment for opacity
      Private Shared _fade As Double
      
      'this value will be used as the timer tick for the fadetimer
      Private Shared _fadetimerinterval As Integer
      
      'the holder for status text that's to be displayed at the bottom of the splash screen
        Private Shared _statusText As String = Nothing
      
      'how long to keep the window open for
      Private Shared _timeout As Integer
      
      'a reference to the form itself
      Private Shared _splashForm As SplashForm = Nothing
      Private pictureBox1 As PictureBox
      
      'label for the starter kit text
      'used for multi threading
      Private Shared _splashthread As Thread = Nothing
      
      'a flag to determine if the form has a dialog box showing
      Private Shared _showDialogBox As Boolean = False
      
      
      
      
      '/ <summary>
      '/ 
      '/ </summary>
      '/ <param name="timeout">
      '/ the number of ms before the form automatically closes.  Pass in 0 to disable this.
      '/ </param>
      Public Sub New(timeout As Integer)
         '
         ' Required for Windows Form Designer support
         '
         InitializeComponent()
         
         timer = New System.Windows.Forms.Timer()
         
         
         'don't start the timer is 0 passed in
         If timeout > 0 Then
            timer.Start()
            timer.Interval = timeout
            AddHandler timer.Tick, AddressOf closeSplash
         End If
         
         fadetimer = New System.Windows.Forms.Timer()
         
         fadetimer.Interval = SplashForm._fadetimerinterval
         
         
         AddHandler fadetimer.Tick, AddressOf fadeTimerHandler
         fadetimer.Start()
         
         
         Me.Opacity = 0
         
         Me.TopLevel = True
         
         Me.ShowInTaskbar = False
            Me.FormBorderStyle = Windows.Forms.FormBorderStyle.None
         Me.StartPosition = FormStartPosition.CenterScreen
      End Sub 'New
      
      
      '/ <summary>
      '/ This is used to show the splash form using a seperate thread.  
      '/ This makes it possible to load in the background on the main thread
      '/ and update your user with status on the splash screen
      '/ </summary>
      '/ <param name="timeout">The number of ms before the form automatically closes.  Pass in 0 to disable this and close manually.</param>
      '/ <param name="fade">A  value representing the change in opacity to make for each tick of the faderTimer.</param>
      '/ <param name="fadertimerinterval">The interval in ms for the fader timer.  Each tick of the fader timer changes the opacity by the amount specified in "fade"</param>
      Public Shared Sub Showasyncsplash(timeout As Integer, fade As Double, fadertimerinterval As Integer)
         SplashForm._timeout = timeout
         SplashForm._fade = fade
         SplashForm._fadetimerinterval = fadertimerinterval
         
         'make sure we only launch the form once
         If _splashForm Is Nothing Then
            _splashthread = New Thread(New ThreadStart(AddressOf SplashForm.ShowSplash))
            _splashthread.IsBackground = True
            _splashthread.SetApartmentState(ApartmentState.STA)
            _splashthread.Start()
         Else
            Return
         End If
      End Sub 'Showasyncsplash
      
      
      Public Shared Sub ShowDialogBox(message As String)
         _showDialogBox = True
         MessageBox.Show(message)
         _showDialogBox = False
         closeAsyncSplash()
      End Sub 'ShowDialogBox
      
      
      Public Shared Sub closeAsyncSplash()
         If Not _showDialogBox Then
            If Not (_splashForm Is Nothing) AndAlso _splashForm.IsDisposed = False AndAlso SplashForm._fade > 0 Then
               'if the timer expires then reverse the fade constant
               SplashForm._fade = - SplashForm._fade
            End If
            
            _splashthread = Nothing
            _splashForm = Nothing
         End If
      End Sub 'closeAsyncSplash
      
      
      '/ <summary>
      '/ shoow the splash form on the new thread
      '/ </summary>
      Public Shared Sub ShowSplash()
         _splashForm = New SplashForm(SplashForm._timeout)
         
         Application.Run(_splashForm)
      End Sub 'ShowSplash
      
      '/ <summary>
      '/ set the private variable to the status text
      '/ </summary>
      '/ <param name="text">The status text to be displayed.</param>
      
      Public Shared Property StatusText() As String
         Get
                Return SplashForm._statusText
         End Get
         Set
                SplashForm._statusText = Value
         End Set
      End Property
      
      '/ <summary>
      '/ return a reference of the splash form
      '/ </summary>
      
      Public Shared ReadOnly Property FormRef() As Form
         Get
            Return _splashForm
         End Get
      End Property
      
      
      '/ <summary>
      '/ Override the OnPaint event to draw the background
      '/ </summary>
      '/ <param name="pea">The OnPaint event arguments</param>
      Protected Overrides Sub OnPaint(pea As PaintEventArgs)
         
            DrawSplashFooterText(_statusText, pea.Graphics, ClientSize.Width, ClientSize.Height)
      End Sub 'OnPaint
      
      
      '/ <summary>
      '/ draws a gradient text string based on the arguments from the OnPaint event
      '/ </summary>
      '/ <param name="grfx">grfx from the onpaint event</param>
      '/ <param name="cx">form width</param>
      '/ <param name="cy">form height</param>
      Private Shared Sub DrawSplashFooterText([text] As String, grfx As Graphics, cx As Integer, cy As Integer)
         'if there is no text to draw then return
         If [text] Is Nothing Then
            Return
         End If
         
         Dim font As New Font("Tahoma", 12, FontStyle.Bold)
         Dim sizef As SizeF = grfx.MeasureString([text], font)
            Dim ptf As New PointF((cx - sizef.Width) / 2, (cy + My.Resources.Resource1.ssk_logo.Height) / 2)
         
         Dim rectf As New RectangleF(ptf, sizef)
         
         Dim lgbrush As New LinearGradientBrush(rectf, Color.Red, Color.Orange, LinearGradientMode.ForwardDiagonal)
         
         grfx.DrawString([text], font, lgbrush, ptf)
         font.Dispose()
      End Sub 'DrawSplashFooterText
       
      
      
      
      
      
      '/ <summary>
      '/ Clean up any resources being used.
      '/ </summary>
      Protected Overrides Sub Dispose(disposing As Boolean)
         If disposing Then
            If Not (components Is Nothing) Then
               components.Dispose()
            End If
         End If
         MyBase.Dispose(disposing)
      End Sub 'Dispose
      
      #Region "Windows Form Designer generated code"
      
      '/ <summary>
      '/ Required method for Designer support - do not modify
      '/ the contents of this method with the code editor.
      '/ </summary>
      Private Sub InitializeComponent()
         Me.pictureBox1 = New System.Windows.Forms.PictureBox()
         CType(Me.pictureBox1, System.ComponentModel.ISupportInitialize).BeginInit()
         Me.SuspendLayout()
         ' 
         ' pictureBox1
         ' 
            Me.pictureBox1.Image = My.Resources.Resource1.ssk_logo
         Me.pictureBox1.Location = New System.Drawing.Point(204, 183)
         Me.pictureBox1.Name = "pictureBox1"
         Me.pictureBox1.Size = New System.Drawing.Size(149, 64)
         Me.pictureBox1.TabIndex = 0
         Me.pictureBox1.TabStop = False
         ' 
         ' SplashForm
         ' 
         Me.BackColor = System.Drawing.Color.Silver
            Me.BackgroundImage = My.Resources.Resource1.splash
         Me.ClientSize = New System.Drawing.Size(567, 441)
         Me.Controls.Add(pictureBox1)
         Me.Name = "SplashForm"
         Me.ShowInTaskbar = False
         Me.Text = "Shareware Starter Kit"
         CType(Me.pictureBox1, System.ComponentModel.ISupportInitialize).EndInit()
         Me.ResumeLayout(False)
      End Sub 'InitializeComponent 
      #End Region
      
      
      
      
      '/ <summary>
      '/ This is the event handler for the timer.
      '/ It closes the splash form and stops the event timer.
      '/ </summary>
      '/ <param name="myObject">event object</param>
      '/ <param name="myEventArgs">Event args</param>
      Private Sub closeSplash(myObject As [Object], myEventArgs As EventArgs)
         
         timer.Stop()
         Me.Close()
      End Sub 'closeSplash
       
      
      '/ <summary>
      '/ This is the event handler for the Fadetimer.
      '/ It changes the opacity of the form
      '/ </summary>
      '/ <param name="myObject">event object</param>
      '/ <param name="myEventArgs">Event args</param>
      Private Sub fadeTimerHandler(myObject As [Object], myEventArgs As EventArgs)
         
         Me.Opacity += _fade
         
         If Me.Opacity <= 0 Then
            Me.Close()
         End If
      End Sub 'fadeTimerHandler
       
      
      
      '/ <summary>
      '/ This event catches the splash form closing event, and changes the sign
      '/ of the fade constant and cancels the close event.  When the fade constant reaches 0 (completely transparent), the form will
      '/ be allowed to close
      '/ </summary>
      '/ <param name="sender"></param>
      '/ <param name="e"></param>
      Sub SplashForm_Closing(sender As Object, e As CancelEventArgs) Handles MyBase.Closing
         'don't close the form if there is a dialog box showing
         If _showDialogBox Then
            
            e.Cancel = True
            Return
         End If
         
         'if the timer expires then reverse the fade constant
         If _fade > 0 Then
            _fade = - _fade
            e.Cancel = True
         End If
      End Sub 'SplashForm_Closing
      
      
      
      
      Private Sub SplashForm_Load(sender As Object, e As System.EventArgs) Handles MyBase.Load
      End Sub 'SplashForm_Load
   End Class 'SplashForm 
End Namespace 'SSK.SSKClient.SSKForms

